@@ -3,7 +3,7 @@
  *
  * NOTE: this code illustrates how to make a dpi-program.
  *
- * Copyright 2002 Jorge Arellano Cid <jcid@dillo.org>
+ * Copyright 2002-2004 Jorge Arellano Cid <jcid@dillo.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,6 +34,7 @@
 #include <netdb.h>
 #include <fcntl.h>
 #include <signal.h>
+#include "dpiutil.h"
 
 #include <glib.h>
 
@@ -75,7 +76,7 @@ static int MODIFY_PAGE_NUM = 1;
 /*
  * Forward declarations
  */
-int send_bm_page(int OutFD);
+int send_bm_page(SockHandler *sh);
 
 
 /* -- HTML templates ------------------------------------------------------- */
@@ -85,7 +86,7 @@ char *mainpage_header =
 "<head>\n"
 "<title>Bookmarks</title>\n"
 "</head>\n"
-"<body bgcolor='#778899' link='black' vlink='gray'>\n"
+"<body bgcolor='#778899' link='black' vlink='brown'>\n"
 "<table border='1' cellpadding='0' width='100%'>\n"
 " <tr><td>\n"
 "  <table width='100%' bgcolor='#b4b4b4'>\n"
@@ -103,7 +104,7 @@ char *modifypage_header =
 "<head>\n"
 "<title>Bookmarks</title>\n"
 "</head>\n"
-"<body bgcolor='#778899' link='black' vlink='gray'>\n"
+"<body bgcolor='#778899' link='black' vlink='brown'>\n"
 "<table border='1' cellpadding='0' width='100%'>\n"
 " <tr><td>\n"
 "  <table width='100%' bgcolor='#b4b4b4'>\n"
@@ -137,7 +138,7 @@ char *mainpage_sections_header =
 "<table border='1' cellpadding='0' cellspacing='20' width='100%'>\n"
 " <tr valign='top'>\n"
 "  <td>\n"
-"   <table bgcolor='#b4b4b4' border='1' cellpadding='4' cellspacing='1'>\n"
+"   <table bgcolor='#b4b4b4' border='2' cellpadding='4' cellspacing='1'>\n"
 "    <tr><td>\n"
 "     <table width='100%' bgcolor='#b4b4b4'>\n"
 "      <tr><td><small>Sections:</small></td></tr></table></td></tr>\n";
@@ -147,20 +148,20 @@ char *modifypage_sections_header =
 " <tr valign='top'>\n"
 "  <td>\n"
 "   <table bgcolor='#b4b4b4' border='1'>\n"
-"    <tr><td>\n"                                           
+"    <tr><td>\n"
 "     <table width='100%' bgcolor='#b4b4b4'>\n"
 "      <tr><td><small>Sections:</small></td></tr></table></td></tr>\n";
 
 char *mainpage_sections_item =
-"    <tr><td>\n"
-"     <table width='100%%' bgcolor='#b4b4b4'>\n"
-"      <tr><td><b><a href='#s%d'>%s</a></b></td></tr></table></td></tr>\n";
+"    <tr><td align='center'>\n"
+"      <a href='#s%d'>%s</a></td></tr>\n";
 
 char *modifypage_sections_item =
 "    <tr><td>\n"
-"     <table width='100%%' bgcolor='#b4b4b4'>\n"
-"      <tr><td width='1%%'><input type='checkbox' name='s%d'></td>\n"
-"       <td><b><a href='#s%d'>%s</a></b></td></tr></table></td></tr>\n";
+"     <table width='100%%' bgcolor='#b4b4b4'cellspacing='0' cellpadding='0'>\n"
+"      <tr align='center'>"
+"       <td width='1%%'><input type='checkbox' name='s%d'></td>\n"
+"       <td><a href='#s%d'>%s</a></td></tr></table></td></tr>\n";
 
 char *mainpage_sections_footer =
 "   </table>\n";
@@ -230,7 +231,7 @@ char *modifypage_add_section_page =
 "<head>\n"
 "<title>Bookmarks</title>\n"
 "</head>\n"
-"<body bgcolor='#778899' link='black' vlink='gray'>\n"
+"<body bgcolor='#778899' link='black' vlink='brown'>\n"
 "<table border='1' cellpadding='0' width='100%'>\n"
 " <tr><td colspan='2'>\n"
 "  <table bgcolor='#b4b4b4' width='100%'>\n"
@@ -264,12 +265,12 @@ char *modifypage_add_section_page =
 "\n";
 
 /* ------------------------------------------------------------------------- */
-char *modifypage_update_header = 
+char *modifypage_update_header =
 "<html>\n"
 "<head>\n"
 "<title>Bookmarks</title>\n"
 "</head>\n"
-"<body bgcolor='#778899' link='black' vlink='gray'>\n"
+"<body bgcolor='#778899' link='black' vlink='brown'>\n"
 "<table border='1' cellpadding='0' width='100%'>\n"
 " <tr><td colspan='2'>\n"
 "  <table bgcolor='#b4b4b4' width='100%'>\n"
@@ -331,7 +332,7 @@ char *modifypage_add_url =
 "<head>\n"
 "<title>Bookmarks</title>\n"
 "</head>\n"
-"<body bgcolor='#778899' link='black' vlink='gray'>\n"
+"<body bgcolor='#778899' link='black' vlink='brown'>\n"
 "<table border='1' cellpadding='0' width='100%'>\n"
 " <tr><td colspan='2'>\n"
 "  <table bgcolor='#b4b4b4' width='100%'>\n"
@@ -400,10 +401,10 @@ char *make_one_line_str(char *str)
 /*
  * Use character stuffing for characters given in 'esc_set'.
  * Return value: New stuffed string.
- */        
+ */
 gchar *Stuff_chars(const gchar *str, const gchar *esc_set)
-{          
-   gint i;             
+{
+   gint i;
    gchar *p, *sstr = NULL;
 
    if ((p = strpbrk(str, esc_set))) {
@@ -412,7 +413,7 @@ gchar *Stuff_chars(const gchar *str, const gchar *esc_set)
       for (i = p - str; gstr->str[i]; ++i)
          if (strchr(esc_set, gstr->str[i])) {
             g_string_insert_c(gstr, i, gstr->str[i]);
-            ++i;   
+            ++i;
          }
       sstr = gstr->str;
       g_string_free(gstr, FALSE);
@@ -420,35 +421,6 @@ gchar *Stuff_chars(const gchar *str, const gchar *esc_set)
    return (sstr ? sstr : g_strdup(str));
 }
 
-/*
- * Task: given a tag and an attribute name, return its value.
- *       (character stuffing is removed here)
- * Return value: the attribute value, or NULL if not present or malformed.
- */          
-char *Get_attr_value(char *tag, int tagsize, char *attrname)
-{            
-   char *p, *q, *ltag, quote, *start, *val = NULL;
-
-   ltag = g_strndup(tag, tagsize);
-   if ((p = strstr(ltag, attrname)) &&
-       (p = strchr(p, '=')) &&
-       (p = strpbrk(p, "'\"")) ) {
-      quote = *p;                 
-      start = ++p;
-      while ((q = strchr(p, quote)) && q[1] == quote)
-         p = q + 2;
-      if (q) {
-         val = g_strndup(start, q - start);
-         for (p = q = val; (*q = *p); ++p, ++q)
-            if ((*p == '"' || *p == '\'') && p[1] == p[0])
-               ++p;
-      }
-   }         
-   g_free(ltag);
-
-   return val;
-}
-
 /*
  * Given an urlencoded string, return it to the original version.
  */
@@ -464,7 +436,7 @@ void Unencode_str(char *e_str)
             *p = '\n';
             e += 5;
          } else {
-            *p = (isdigit(e[1]) ? (e[1] - '0') : (e[1] - 'A' + 10)) * 16 + 
+            *p = (isdigit(e[1]) ? (e[1] - '0') : (e[1] - 'A' + 10)) * 16 +
                  (isdigit(e[2]) ? (e[2] - '0') : (e[2] - 'A' + 10));
             e += 2;
          }
@@ -506,38 +478,17 @@ char *Get_line(FILE *stream)
    return buf;
 }
 
-/*
- * Send a sized data stream to client.
- */
-int send_data_chunk(int OutFD, char *Data, size_t DataSize)
-{
-   if (send(OutFD, Data, DataSize, 0) == -1) {
-      perror("[send_data_chunk]");
-      return 1;
-   }
-   //sleep(1);
-   return 0;
-}
-
-/*
- * Send a data string to client.
- */
-int send_data_string(int OutFD, char *str)
-{
-   return send_data_chunk(OutFD, str, strlen(str));
-}
-
 /*
  * Send a short message to dillo's status bar.
  */
-int Bmsrv_dpi_send_status_msg(int OutFD, char *str)
+int Bmsrv_dpi_send_status_msg(SockHandler *sh, char *str)
 {
    int st;
    GString *gstr = g_string_new("");
    gchar *sstr = Stuff_chars(str, "'\"");
 
    g_string_sprintf(gstr, "<dpi cmd='send_status_message' msg='%s'>", sstr);
-   st = send_data_chunk(OutFD, gstr->str, gstr->len);
+   st = sock_handler_write(sh, gstr->str, gstr->len, 1);
    g_string_free(gstr, TRUE);
    g_free(sstr);
    return st;
@@ -782,7 +733,7 @@ void Bms_normalize()
 void Bms_check_import()
 {
    gchar *OldBmFile;
-   char *cmd1 = 
+   char *cmd1 =
       "echo \":s0: Unclassified\" > %s";
    char *cmd2 =
       "grep -i \"href\" %s | "
@@ -949,7 +900,7 @@ int Bms_save()
 /*
  * Add a new bookmark to DB :)
  */
-int Bmsrv_add_bm(int SockFD, char *url, char *title)
+int Bmsrv_add_bm(SockHandler *sh, char *url, char *title)
 {
    char *msg="Added bookmark!";
    int section = 0;
@@ -961,7 +912,7 @@ int Bmsrv_add_bm(int SockFD, char *url, char *title)
    /* Write to file */
    Bms_save();
 
-   if (Bmsrv_dpi_send_status_msg(SockFD, msg))
+   if (Bmsrv_dpi_send_status_msg(sh, msg))
       return 1;
 
    return 0;
@@ -995,13 +946,13 @@ static void Bmsrv_count_urls_and_sections(char *url, gint *n_sec, gint *n_url)
  * Send a dpi reload request
  * Return code: { 0:OK, 1:Abort, 2:Close }
  */
-static int Bmsrv_send_reload_request(int OutFD, char *url)
+static int Bmsrv_send_reload_request(SockHandler *sh, char *url)
 {
    gint st = 0;
    GString *gstr = g_string_new("");
 
    g_string_sprintf(gstr, "<dpi cmd='reload_request' url='%s'>", url);
-   if (send_data_chunk(OutFD, gstr->str, gstr->len))
+   if (sock_handler_write(sh, gstr->str, gstr->len, 1))
       st = 1;
    g_string_free(gstr, TRUE);
 
@@ -1012,7 +963,7 @@ static int Bmsrv_send_reload_request(int OutFD, char *url)
  * Send the HTML for the modify page
  * Return code: { 0:OK, 1:Abort, 2:Close }
  */
-static int Bmsrv_send_modify_page(int OutFD)
+static int Bmsrv_send_modify_page(SockHandler *sh)
 {
    static GString *gstr = NULL;
    gchar *l_title;
@@ -1026,26 +977,26 @@ static int Bmsrv_send_modify_page(int OutFD)
       gstr = g_string_new("");
 
    /* send modify page header */
-   if (send_data_string(OutFD, modifypage_header))
+   if (sock_handler_write_str(sh, modifypage_header, 0))
       return 1;
 
    /* write sections header */
-   if (send_data_string(OutFD, modifypage_sections_header))
+   if (sock_handler_write_str(sh, modifypage_sections_header, 0))
       return 1;
    /* write sections */
    for (list1 = B_secs; list1; list1 = list1->next) {
       sec_node = list1->data;
       g_string_sprintf(gstr, modifypage_sections_item,
                        sec_node->section, sec_node->section, sec_node->title);
-      if (send_data_string(OutFD, gstr->str))
+      if (sock_handler_write_str(sh, gstr->str, 0))
          return 1;
    }
    /* write sections footer */
-   if (send_data_string(OutFD, modifypage_sections_footer))
+   if (sock_handler_write_str(sh, modifypage_sections_footer, 0))
       return 1;
 
    /* send page middle */
-   if (send_data_string(OutFD, modifypage_middle1))
+   if (sock_handler_write_str(sh, modifypage_middle1, 0))
       return 1;
 
    /* send bookmark cards */
@@ -1057,7 +1008,7 @@ static int Bmsrv_send_modify_page(int OutFD)
       g_string_sprintf(gstr, modifypage_section_card_header,
                        sec_node->section, l_title);
       g_free(l_title);
-      if (send_data_string(OutFD, gstr->str))
+      if (sock_handler_write_str(sh, gstr->str, 0))
          return 1;
 
       /* send section's bookmarks */
@@ -1066,18 +1017,18 @@ static int Bmsrv_send_modify_page(int OutFD)
          if (bm_node->section == sec_node->section) {
             g_string_sprintf(gstr, modifypage_section_card_item,
                              bm_node->key, bm_node->url, bm_node->title);
-            if (send_data_string(OutFD, gstr->str))
+            if (sock_handler_write_str(sh, gstr->str, 0))
                return 1;
          }
       }
 
       /* send card footer */
-      if (send_data_string(OutFD, modifypage_section_card_footer))
+      if (sock_handler_write_str(sh, modifypage_section_card_footer, 0))
          return 1;
    }
 
    /* finish page */
-   if (send_data_string(OutFD, modifypage_footer))
+   if (sock_handler_write_str(sh, modifypage_footer, 1))
       return 1;
 
    //g_print("  page body sent.\n");
@@ -1088,12 +1039,12 @@ static int Bmsrv_send_modify_page(int OutFD)
  * Send the HTML for the modify page for "add section"
  * Return code: { 0:OK, 1:Abort, 2:Close }
  */
-static int Bmsrv_send_modify_page_add_section(int OutFD)
+static int Bmsrv_send_modify_page_add_section(SockHandler *sh)
 {
    //g_print("  sending page body...\n");
 
    /* send modify page2 */
-   if (send_data_string(OutFD, modifypage_add_section_page))
+   if (sock_handler_write_str(sh, modifypage_add_section_page, 1))
       return 1;
 
    //g_print("  page body sent.\n");
@@ -1104,21 +1055,21 @@ static int Bmsrv_send_modify_page_add_section(int OutFD)
  * Send the HTML for the modify page for "add url"
  * Return code: { 0:OK, 1:Abort, 2:Close }
  */
-static int Bmsrv_send_modify_page_add_url(int OutFD)
+static int Bmsrv_send_modify_page_add_url(SockHandler *sh)
 {
-   if (send_data_string(OutFD, modifypage_add_url))
+   if (sock_handler_write_str(sh, modifypage_add_url, 1))
       return 1;
    return 2;
 }
 
 /*
  * Parse a modify urls request and either:
- *   - make a local copy of the url 
+ *   - make a local copy of the url
  *     or
  *   - send the modify page for the marked urls and sections
  * Return code: { 0:OK, 1:Abort, 2:Close }
  */
-static int Bmsrv_send_modify_update(int OutFD, char *url)
+static int Bmsrv_send_modify_update(SockHandler *sh, char *url)
 {
    static char *url1 = NULL;
    static GString *gstr = NULL;
@@ -1132,7 +1083,7 @@ static int Bmsrv_send_modify_update(int OutFD, char *url)
    if (!gstr)
       gstr = g_string_new("");
 
-   if (OutFD == -1) {
+   if (sh == NULL) {
       /* just copy url */
       g_free(url1);
       url1 = g_strdup(url);
@@ -1140,7 +1091,7 @@ static int Bmsrv_send_modify_update(int OutFD, char *url)
    }
 
    /* send HTML here */
-   if (send_data_string(OutFD, modifypage_update_header))
+   if (sock_handler_write_str(sh, modifypage_update_header, 0))
       return 1;
 
    /* Count number of marked urls and sections */
@@ -1148,8 +1099,8 @@ static int Bmsrv_send_modify_update(int OutFD, char *url)
 
    if (n_sec) {
       g_string_sprintf(gstr, modifypage_update_title, "Update&nbsp;sections:");
-      send_data_chunk(OutFD, gstr->str, gstr->len);
-      send_data_string(OutFD, modifypage_update_item_header);
+      sock_handler_write(sh, gstr->str, gstr->len, 0);
+      sock_handler_write_str(sh, modifypage_update_item_header, 0);
       /* send items here */
       p = strchr(url1, '?');
       for (q = p; (q = strstr(q, "&s")); ++q) {
@@ -1159,17 +1110,17 @@ static int Bmsrv_send_modify_update(int OutFD, char *url)
             if ((sec_node = Bms_get_sec(key))) {
                g_string_sprintf(gstr, modifypage_update_item2,
                                 sec_node->section, sec_node->title);
-               send_data_chunk(OutFD, gstr->str, gstr->len);
+               sock_handler_write(sh, gstr->str, gstr->len, 0);
             }
          }
       }
-      send_data_string(OutFD, modifypage_update_item_footer);
+      sock_handler_write_str(sh, modifypage_update_item_footer, 0);
    }
 
    if (n_url) {
       g_string_sprintf(gstr, modifypage_update_title, "Update&nbsp;titles:");
-      send_data_chunk(OutFD, gstr->str, gstr->len);
-      send_data_string(OutFD, modifypage_update_item_header);
+      sock_handler_write(sh, gstr->str, gstr->len, 0);
+      sock_handler_write_str(sh, modifypage_update_item_header, 0);
       /* send items here */
       p = strchr(url1, '?');
       for (q = p; (q = strstr(q, "&url")); ++q) {
@@ -1179,13 +1130,13 @@ static int Bmsrv_send_modify_update(int OutFD, char *url)
             bm_node = Bms_get(key);
             g_string_sprintf(gstr, modifypage_update_item,
                              bm_node->key, bm_node->title, bm_node->url);
-            send_data_chunk(OutFD, gstr->str, gstr->len);
+            sock_handler_write(sh, gstr->str, gstr->len, 0);
          }
       }
-      send_data_string(OutFD, modifypage_update_item_footer);
+      sock_handler_write_str(sh, modifypage_update_item_footer, 0);
    }
 
-   send_data_string(OutFD, modifypage_update_footer);
+   sock_handler_write_str(sh, modifypage_update_footer, 1);
 
    return 2;
 }
@@ -1194,38 +1145,36 @@ static int Bmsrv_send_modify_update(int OutFD, char *url)
  * Make the modify-page and send it back
  * Return code: { 0:OK, 1:Abort, 2:Close }
  */
-static int Bmsrv_send_modify_answer(int SockFD, char *url)
+static int Bmsrv_send_modify_answer(SockHandler *sh, char *url)
 {
    GString *gstr = g_string_new("");
 
    g_string_sprintf(gstr, "<dpi cmd='start_send_page' url='%s'>", url);
    //g_print("  cmd_buf=%s\n", gstr->str);
    /* Send dpi command */
-   if (send(SockFD, gstr->str, gstr->len, 0) == -1) {
+   if (sock_handler_write(sh, gstr->str, gstr->len, 1) != 0) {
       g_string_free(gstr, TRUE);
-      perror("[send]");
       return 1;
    }
    g_string_free(gstr, TRUE);
 
    /* Send HTTP header */
    //g_print("  sending Header...\n");
-   if (send(SockFD, Header, strlen(Header), 0) == -1) {
-      perror("[send]");
+   if (sock_handler_write(sh, Header, strlen(Header), 0) != 0) {
       return 1;
    }
 
    if (MODIFY_PAGE_NUM == 2) {
       MODIFY_PAGE_NUM = 1;
-      return Bmsrv_send_modify_page_add_section(SockFD);
+      return Bmsrv_send_modify_page_add_section(sh);
    } else if (MODIFY_PAGE_NUM == 3) {
       MODIFY_PAGE_NUM = 1;
-      return Bmsrv_send_modify_update(SockFD, NULL);
+      return Bmsrv_send_modify_update(sh, NULL);
    } else if (MODIFY_PAGE_NUM == 4) {
       MODIFY_PAGE_NUM = 1;
-      return Bmsrv_send_modify_page_add_url(SockFD);
+      return Bmsrv_send_modify_page_add_url(sh);
    } else {
-      return Bmsrv_send_modify_page(SockFD);
+      return Bmsrv_send_modify_page(sh);
    }
 }
 
@@ -1236,7 +1185,7 @@ static int Bmsrv_send_modify_answer(int SockFD, char *url)
  * Parse a delete bms request, delete them, and update bm file.
  * Return code: { 0:OK, 1:Abort }
  */
-static int Bmsrv_modify_delete(int OutFD, char *url)
+static int Bmsrv_modify_delete(SockHandler *sh, char *url)
 {
    gchar *p;
    gint nb, ns, key;
@@ -1267,7 +1216,7 @@ static int Bmsrv_modify_delete(int OutFD, char *url)
  *    receipt of the first data stream.
  *
    sprintf(msg, "Deleted %d bookmark%s!>", n, (n > 1) ? "s" : "");
-   if (Bmsrv_dpi_send_status_msg(OutFD, msg))
+   if (Bmsrv_dpi_send_status_msg(sh, msg))
       return 1;
 */
 
@@ -1282,7 +1231,7 @@ static int Bmsrv_modify_delete(int OutFD, char *url)
  * Parse a move urls request, move and update bm file.
  * Return code: { 0:OK, 1:Abort }
  */
-static int Bmsrv_modify_move(int OutFD, char *url)
+static int Bmsrv_modify_move(SockHandler *sh, char *url)
 {
    char *p;
    int n, section = 0, key;
@@ -1321,7 +1270,7 @@ static int Bmsrv_modify_move(int OutFD, char *url)
  * Parse a modify request: update urls and sections, then save.
  * Return code: { 0:OK, 1:Abort }
  */
-static int Bmsrv_modify_update(int OutFD, char *url)
+static int Bmsrv_modify_update(SockHandler *sh, char *url)
 {
    char *p, *q, *title;
    int i, key;
@@ -1376,7 +1325,7 @@ static int Bmsrv_modify_update(int OutFD, char *url)
  * Parse an "add section" request, and update the bm file.
  * Return code: { 0:OK, 1:Abort }
  */
-static int Bmsrv_modify_add_section(int OutFD, char *url)
+static int Bmsrv_modify_add_section(SockHandler *sh, char *url)
 {
    char *p, *title = NULL;
 
@@ -1404,7 +1353,7 @@ static int Bmsrv_modify_add_section(int OutFD, char *url)
  * Parse an "add url" request, and update the bm file.
  * Return code: { 0:OK, 1:Abort }
  */
-static int Bmsrv_modify_add_url(int OutFD, char *s_url)
+static int Bmsrv_modify_add_url(SockHandler *sh, char *s_url)
 {
    char *p, *q, *title, *url;
    int i;
@@ -1412,7 +1361,7 @@ static int Bmsrv_modify_add_url(int OutFD, char *s_url)
 
    /* bookmarks were loaded before */
 
-   if (OutFD == -1) {
+   if (sh == NULL) {
       /* look for section */
       for (q = s_url; (q = strstr(q, "&s")); ++q) {
          for (i = 0; isdigit(q[2+i]); ++i);
@@ -1455,7 +1404,7 @@ static int Bmsrv_modify_add_url(int OutFD, char *s_url)
  * when it's wrong.
  * Return code: { 0:OK, 2:Close }
  */
-static int Bmsrv_check_modify_request(int OutFD, char *url)
+static int Bmsrv_check_modify_request(SockHandler *sh, char *url)
 {
    char *p, *msg;
    int n_sec, n_url;
@@ -1512,7 +1461,7 @@ static int Bmsrv_check_modify_request(int OutFD, char *url)
       msg = "Sorry, not implemented yet.";
    }
 
-   Bmsrv_dpi_send_status_msg(OutFD, msg);
+   Bmsrv_dpi_send_status_msg(sh, msg);
    return 2;
 }
 
@@ -1520,61 +1469,61 @@ static int Bmsrv_check_modify_request(int OutFD, char *url)
  * Parse a and process a modify request.
  * Return code: { 0:OK, 1:Abort, 2:Close }
  */
-static int Bmsrv_process_modify_request(int OutFD, char *url)
+static int Bmsrv_process_modify_request(SockHandler *sh, char *url)
 {
    /* check the provided parameters */
-   if (Bmsrv_check_modify_request(OutFD, url) != 0)
+   if (Bmsrv_check_modify_request(sh, url) != 0)
       return 2;
 
    if (strstr(url, "operation=delete&")) {
-      if (Bmsrv_modify_delete(OutFD, url) == 1)
+      if (Bmsrv_modify_delete(sh, url) == 1)
          return 1;
-      if (Bmsrv_send_reload_request(OutFD, "dpi:/bm/modify") == 1)
+      if (Bmsrv_send_reload_request(sh, "dpi:/bm/modify") == 1)
          return 1;
 
    } else if (strstr(url, "operation=move&")) {
-      if (Bmsrv_modify_move(OutFD, url) == 1)
+      if (Bmsrv_modify_move(sh, url) == 1)
          return 1;
-      if (Bmsrv_send_reload_request(OutFD, "dpi:/bm/modify") == 1)
+      if (Bmsrv_send_reload_request(sh, "dpi:/bm/modify") == 1)
          return 1;
 
    } else if (strstr(url, "operation=modify&")) {
       /* make a local copy of 'url' */
-      Bmsrv_send_modify_update(-1, url);
+      Bmsrv_send_modify_update(NULL, url);
       MODIFY_PAGE_NUM = 3;
-      if (Bmsrv_send_reload_request(OutFD, "dpi:/bm/modify") == 1)
+      if (Bmsrv_send_reload_request(sh, "dpi:/bm/modify") == 1)
          return 1;
 
    } else if (strstr(url, "operation=modify2&")) {
-      if (Bmsrv_modify_update(OutFD, url) == 1)
+      if (Bmsrv_modify_update(sh, url) == 1)
          return 1;
-      if (Bmsrv_send_reload_request(OutFD, "dpi:/bm/modify") == 1)
+      if (Bmsrv_send_reload_request(sh, "dpi:/bm/modify") == 1)
          return 1;
 
    } else if (strstr(url, "operation=add_sec&")) {
       /* this global variable tells which page to send  (--hackish...) */
       MODIFY_PAGE_NUM = 2;
-      if (Bmsrv_send_reload_request(OutFD, "dpi:/bm/modify") == 1)
+      if (Bmsrv_send_reload_request(sh, "dpi:/bm/modify") == 1)
          return 1;
 
    } else if (strstr(url, "operation=add_section&")) {
-      if (Bmsrv_modify_add_section(OutFD, url) == 1)
+      if (Bmsrv_modify_add_section(sh, url) == 1)
          return 1;
-      if (Bmsrv_send_reload_request(OutFD, "dpi:/bm/modify") == 1)
+      if (Bmsrv_send_reload_request(sh, "dpi:/bm/modify") == 1)
          return 1;
 
    } else if (strstr(url, "operation=add_url&")) {
       /* this global variable tells which page to send  (--hackish...) */
       MODIFY_PAGE_NUM = 4;
       /* parse section if present */
-      Bmsrv_modify_add_url(-1, url);
-      if (Bmsrv_send_reload_request(OutFD, "dpi:/bm/modify") == 1)
+      Bmsrv_modify_add_url(NULL, url);
+      if (Bmsrv_send_reload_request(sh, "dpi:/bm/modify") == 1)
          return 1;
 
    } else if (strstr(url, "operation=add_url2&")) {
-      if (Bmsrv_modify_add_url(OutFD, url) == 1)
+      if (Bmsrv_modify_add_url(sh, url) == 1)
          return 1;
-      if (Bmsrv_send_reload_request(OutFD, "dpi:/bm/modify") == 1)
+      if (Bmsrv_send_reload_request(sh, "dpi:/bm/modify") == 1)
          return 1;
    }
 
@@ -1586,7 +1535,7 @@ static int Bmsrv_process_modify_request(int OutFD, char *url)
 /*
  * Send the current bookmarks page (in HTML)
  */
-int send_bm_page(int OutFD)
+int send_bm_page(SockHandler *sh)
 {
    static GString *gstr = NULL;
    gchar *l_title;
@@ -1597,26 +1546,26 @@ int send_bm_page(int OutFD)
    if (!gstr)
       gstr = g_string_new("");
 
-   if (send_data_string(OutFD, mainpage_header))
+   if (sock_handler_write_str(sh, mainpage_header, 0))
       return 1;
 
    /* write sections header */
-   if (send_data_string(OutFD, mainpage_sections_header))
+   if (sock_handler_write_str(sh, mainpage_sections_header, 0))
       return 1;
    /* write sections */
    for (list1 = B_secs; list1; list1 = list1->next) {
       sec_node = list1->data;
       g_string_sprintf(gstr, mainpage_sections_item,
                        sec_node->section, sec_node->title);
-      if (send_data_string(OutFD, gstr->str))
+      if (sock_handler_write_str(sh, gstr->str, 0))
          return 1;
    }
    /* write sections footer */
-   if (send_data_string(OutFD, mainpage_sections_footer))
+   if (sock_handler_write_str(sh, mainpage_sections_footer, 0))
       return 1;
 
    /* send page middle */
-   if (send_data_string(OutFD, mainpage_middle1))
+   if (sock_handler_write_str(sh, mainpage_middle1, 0))
       return 1;
 
    /* send bookmark cards */
@@ -1628,7 +1577,7 @@ int send_bm_page(int OutFD)
       g_string_sprintf(gstr, mainpage_section_card_header,
                        sec_node->section, l_title);
       g_free(l_title);
-      if (send_data_string(OutFD, gstr->str))
+      if (sock_handler_write_str(sh, gstr->str, 0))
          return 1;
 
       /* send section's bookmarks */
@@ -1637,18 +1586,18 @@ int send_bm_page(int OutFD)
          if (bm_node->section == sec_node->section) {
             g_string_sprintf(gstr, mainpage_section_card_item,
                              bm_node->url, bm_node->title);
-            if (send_data_string(OutFD, gstr->str))
+            if (sock_handler_write_str(sh, gstr->str, 0))
                return 1;
          }
       }
 
       /* send card footer */
-      if (send_data_string(OutFD, mainpage_section_card_footer))
+      if (sock_handler_write_str(sh, mainpage_section_card_footer, 0))
          return 1;
    }
 
    /* finish page */
-   if (send_data_string(OutFD, mainpage_footer))
+   if (sock_handler_write_str(sh, mainpage_footer, 1))
       return 1;
 
    return 0;
@@ -1662,7 +1611,7 @@ int send_bm_page(int OutFD)
  * Note: Buf is a zero terminated string
  * Return code: { 0:OK, 1:Abort, 2:Close }
  */
-int Bmsrv_parse_buf(int SockFD, char *Buf, int BufSize)
+int Bmsrv_parse_buf(SockHandler *sh, char *Buf, int BufSize)
 {
    char *msg1="<dpi cmd='chat' msg='Hi browser'>";
    char *msg2="<dpi cmd='chat' msg='Is it worth?'>";
@@ -1685,15 +1634,15 @@ int Bmsrv_parse_buf(int SockFD, char *Buf, int BufSize)
       msg = Get_attr_value(Buf, BufSize, "msg");
       if (*msg == 'H') {
          /* "Hi server" */
-         if (send_data_chunk(SockFD, msg1, strlen(msg1)))
+         if (sock_handler_write_str(sh, msg1, 1))
             return 1;
       } else if (*msg == 'I') {
          /* "I want to set abookmark" */
-         if (send_data_chunk(SockFD, msg2, strlen(msg2)))
+         if (sock_handler_write_str(sh, msg2, 1))
             return 1;
       } else if (*msg == 'S') {
          /* "Sure" */
-         if (send_data_chunk(SockFD, msg3, strlen(msg3)))
+         if (sock_handler_write_str(sh, msg3, 1))
             return 1;
       }
       g_free(msg);
@@ -1716,7 +1665,7 @@ int Bmsrv_parse_buf(int SockFD, char *Buf, int BufSize)
          title = g_strdup("(Untitled)");
       }
       if (url && title)
-         Bmsrv_add_bm(SockFD, url, title);
+         Bmsrv_add_bm(sh, url, title);
       g_free(url);
       g_free(title);
       return 2;
@@ -1726,37 +1675,34 @@ int Bmsrv_parse_buf(int SockFD, char *Buf, int BufSize)
       url = Get_attr_value(Buf, BufSize, "url");
 
       if (strcmp(url, "dpi:/bm/modify") == 0) {
-         st = Bmsrv_send_modify_answer(SockFD, url);
+         st = Bmsrv_send_modify_answer(sh, url);
          return st;
 
       } else if (strncmp(url, "dpi:/bm/modify?", 15) == 0) {
          /* process request */
-         st = Bmsrv_process_modify_request(SockFD, url);
+         st = Bmsrv_process_modify_request(sh, url);
          return st;
       }
 
       sprintf(cmd_buf, "<dpi cmd='start_send_page' url='%s'>", url);
       //g_print("  cmd_buf=%s\n", cmd_buf);
       /* Send dpi command */
-      if (send(SockFD, cmd_buf, strlen(cmd_buf), 0) == -1) {
-         perror("[send]");
+      if (sock_handler_write(sh, cmd_buf, strlen(cmd_buf), 1) != 0) {
          return 1;
       }
 
       /* Send HTTP header */
       //g_print("  sending HTTP header...\n");
-      if (send(SockFD, Header, strlen(Header), 0) == -1) {
-         perror("[send]");
+      if (sock_handler_write(sh, Header, strlen(Header), 1) != 0) {
          return 1;
       }
 
       //g_print("  sending page body...\n");
-      st = send_bm_page(SockFD);
+      st = send_bm_page(sh);
       if (st != 0) {
-         char *err = 
+         char *err =
             "<HTML><body> Error on the bookmarks server...</body></html>";
-         if (send(SockFD, err, strlen(err), 0) == -1) {
-            perror("[send]");
+         if (sock_handler_write(sh, err, strlen(err), 1) != 0) {
             return 1;
          }
       }
@@ -1793,8 +1739,9 @@ int main (void) {
    struct sockaddr_un spun;
    int temp_sock_descriptor;
    int address_size;
-   char buf[16384];
-   int st, code;
+   char *buf;
+   int code;
+   SockHandler *sh;
 
    /* Arrange the cleanup function for terminations via exit() */
    atexit(cleanup);
@@ -1807,9 +1754,6 @@ int main (void) {
    if (signal (SIGTERM, termination_handler) == SIG_IGN)
      signal (SIGTERM, SIG_IGN);
 
-   /* Remove the old socket filename */
-   cleanup();
-
    BmFile = g_strconcat(g_get_home_dir(), "/", ".dillo/bm.txt", NULL);
 
    g_print("bookmarks.dpi (v.13): accepting connections...\n");
@@ -1818,35 +1762,31 @@ int main (void) {
    address_size = sizeof(struct sockaddr_un);
 
    while (1) {
-      temp_sock_descriptor = 
+      temp_sock_descriptor =
          accept(STDIN_FILENO, (struct sockaddr *)&spun, &address_size);
       if (temp_sock_descriptor == -1) {
          perror("[accept]");
          exit(1);
       }
 
+      /* create the SockHandler structure */
+      sh = sock_handler_new(temp_sock_descriptor,temp_sock_descriptor,8*1024);
+
       while (1) {
-         //sleep(1);
-         do
-            st = recv(temp_sock_descriptor, buf, 16384, 0);
-         while (st < 0 && errno == EINTR);
-         if (st == -1) {
-            perror("[recv]");
-            exit(1);
+         code = 1;
+         if ((buf = sock_handler_read(sh)) != NULL) {
+            /* Let's see what we fished... */
+            code = Bmsrv_parse_buf(sh, buf, strlen(buf));
          }
-         buf[st] = 0;
-         //g_print("Received from client: %s\n", buf);
-   
-         /* Let's see what we fished... */
-         code = Bmsrv_parse_buf(temp_sock_descriptor, buf, st);
          if (code == 1)
             exit(1);
          else if (code == 2)
             break;
       }
 
-      //g_print("Closing temp_sock_descriptor\n");
-      close(temp_sock_descriptor);
+      //g_print("Closing SockHandler\n");
+      sock_handler_close(sh);
+      sock_handler_free(sh);
 
    }/*while*/
 }
