@@ -126,7 +126,7 @@ void free_plugin_list(struct dp **dpi_attr_list_ptr, int numdpis)
 }
 
 /*! \todo
- * Remove terminator and est_terminator unless we really want to clean up 
+ * Remove terminator and est_terminator unless we really want to clean up
  * on abnormal exit.
  */
 #if 0
@@ -141,7 +141,7 @@ void terminator(int sig)
    _exit(0);
 }
 
-/*! Establish handler for termination signals 
+/*! Establish handler for termination signals
  * and register cleanup with atexit */
 void est_terminator(void)
 {
@@ -215,6 +215,9 @@ int get_dpi_attr(char *dpi_dir, char *service, struct dp *dpi_attr)
    } else if ( (dir_stream = opendir(service_dir)) == NULL) {
       ERRMSG("get_dpi_attr", "opendir", errno);
    } else {
+      /* Scan the directory loking for dpi files.
+       * (currently there's only the dpi program, but in the future
+       *  there may also be helper scripts.) */
       while ( (dir_entry = readdir(dir_stream)) != NULL) {
          if (dir_entry->d_name[0] == '.')
             continue;
@@ -231,18 +234,17 @@ int get_dpi_attr(char *dpi_dir, char *service, struct dp *dpi_attr)
                   dpi_attr->filter = 1;
                else
                   dpi_attr->filter = 0;
-               break;
-            case UNKNOWN_FILE:
+               retval = 0;
                break;
             default:
-               ERRMSG("Fatal error: get_dpi_attr", "get_file_type", 0);
-               fprintf(stderr, " - switch cases do not match ftype\n");
-               exit(1);
                break;
          }
       }
       closedir(dir_stream);
-      retval = 0;
+
+      if (retval != 0)
+         fprintf(stderr,"get_dpi_attr: No dpi plug-in in %s/%s\n",
+                 dpi_dir, service);
    }
    g_free(service_dir);
    return retval;
@@ -306,7 +308,7 @@ int register_service(struct dp *dpi_attr, char *service)
 }
 
 /*!
- * Create dpi directory for available 
+ * Create dpi directory for available
  * plugins and create plugin list.
  * \Return
  * \li Returns number of available plugins on success
@@ -318,7 +320,7 @@ int register_all(struct dp **attlist)
    char *user_dpidir = NULL, *sys_dpidir = NULL, *dpidrc = NULL;
    char *basename=NULL;
    struct dirent *user_dirent, *sys_dirent;
-   int j, not_in_user_list;
+   int j, st, not_in_user_list;
    int snum, usr_srv_num;
    size_t dp_sz = sizeof(struct dp);
 
@@ -359,7 +361,9 @@ int register_all(struct dp **attlist)
          if (user_dirent->d_name[0] == '.')
             continue;
          *attlist = (struct dp *) g_realloc(*attlist, (snum + 1) * dp_sz);
-         get_dpi_attr(user_dpidir, user_dirent->d_name, &(*attlist)[snum++]);
+         st=get_dpi_attr(user_dpidir, user_dirent->d_name, &(*attlist)[snum]);
+         if (st == 0)
+            snum++;
       }
       usr_srv_num = snum;
       closedir(user_dir_stream);
@@ -368,7 +372,7 @@ int register_all(struct dp **attlist)
       /* if system service is not in user list then add it */
       while ((sys_dirent = readdir(sys_dir_stream)) != NULL) {
          if (sys_dirent->d_name[0] == '.')
-           continue; 
+           continue;
          not_in_user_list = 1;
          for (j = 0; j < usr_srv_num; j++) {
             basename = g_basename((*attlist)[j].path);
@@ -379,7 +383,9 @@ int register_all(struct dp **attlist)
          }
          if (not_in_user_list) {
             *attlist = (struct dp *) g_realloc(*attlist, (snum + 1) * dp_sz);
-            get_dpi_attr(sys_dpidir, sys_dirent->d_name, &(*attlist)[snum++]);
+            st=get_dpi_attr(sys_dpidir, sys_dirent->d_name, &(*attlist)[snum]);
+            if (st == 0)
+               snum++;
          }
       }
       closedir(sys_dir_stream);
@@ -424,7 +430,7 @@ int init_srs_socket(char *sockdir)
       fprintf(stderr, "\n - it should be <= %d chars", sun_path_len);
       fprintf(stderr, "\n - srs_name = %s\n", srs_name);
       return(retval);
-   } 
+   }
    strncpy(srs_sa.sun_path, srs_name, sun_path_len);
    addr_sz = (socklen_t) D_SUN_LEN(&srs_sa);
 
@@ -462,6 +468,7 @@ int init_dpi_socket(struct dp *dpi_attr, char *sockdir)
    struct sockaddr_un sa;
    size_t sp_len;
    socklen_t addr_sz;
+   size_t sock_buflen = 8192;
 
    sp_len = sizeof(sa.sun_path);
    if ((s = socket(AF_LOCAL, SOCK_STREAM, 0)) == -1) {
@@ -471,6 +478,10 @@ int init_dpi_socket(struct dp *dpi_attr, char *sockdir)
    /* Set the socket FD to close on exec */
    fcntl(s, F_SETFD, FD_CLOEXEC | fcntl(s, F_GETFD));
 
+   /* set some buffering to increase the transfer's speed */
+   setsockopt(s, SOL_SOCKET, SO_SNDBUF,
+              &sock_buflen, (socklen_t)sizeof(sock_buflen));
+
    dpi_attr->socket = s;
    dpi_attr->sa.sun_family = AF_LOCAL;
    dpi_nm = g_basename(dpi_attr->path);
@@ -481,7 +492,7 @@ int init_dpi_socket(struct dp *dpi_attr, char *sockdir)
       fprintf(stderr, "\n - it should be <= %d chars", sp_len);
       fprintf(stderr, "\n - socket path = %s\n", dpi_attr->sockpath);
       return(-1);
-   } 
+   }
    strncpy(dpi_attr->sa.sun_path, dpi_attr->sockpath, sp_len);
    addr_sz = (socklen_t) D_SUN_LEN(&dpi_attr->sa);
 
@@ -607,7 +618,7 @@ void stop_active_dpis(struct dp *dpi_attr_list, int numdpis)
          ERRMSG("stop_active_dpis", "socket path is too long", 0);
          fprintf(stderr, "\n - it should be <= %d chars", sun_path_len);
          fprintf(stderr, "\n - socket path = %s\n", dpi_attr_list[i].sockpath);
-      } 
+      }
       strncpy(dpi_addr.sun_path, dpi_attr_list[i].sockpath, sun_path_len);
       if (connect(dpi_socket, (struct sockaddr *) &dpi_addr, addr_len) == -1) {
          ERRMSG("stop_active_dpis", "connect", errno);
