@@ -23,7 +23,6 @@
 #include <sys/stat.h>
 #include <sys/uio.h>
 #include <sys/socket.h>
-#include <glib.h>
 #include <gdk/gdk.h>
 #include "../msg.h"
 #include "../chain.h"
@@ -144,6 +143,8 @@ typedef struct {
    pthread_cond_t cond;
 
    gint FD;
+   void *Key;
+
    void *Buf1Start;
    void *Buf1;
    size_t Buf1Size;
@@ -151,6 +152,7 @@ typedef struct {
    size_t Buf2Size;
 
    gint LastChunk;
+   gint CloseFD;
    gint Done;
 } thr_data_t;
 
@@ -162,7 +164,7 @@ static gint ThrDataMax = 8;
 /*
  * Create a new ThrData node
  */
-static thr_data_t *IO_thrdata_new(gint FD)
+static thr_data_t *IO_thrdata_new(gint FD, void *Key)
 {
    thr_data_t *data;
 
@@ -170,9 +172,11 @@ static thr_data_t *IO_thrdata_new(gint FD)
    pthread_mutex_init(&data->mut, NULL);
    pthread_cond_init(&data->cond, NULL);
    data->FD = FD;
+   data->Key = Key;
    data->Buf1Start = data->Buf1 = data->Buf2 = NULL;
    data->Buf1Size = data->Buf2Size = 0;
    data->LastChunk = 0;
+   data->CloseFD = 0;
    data->Done = 0;
    return data;
 }
@@ -189,21 +193,27 @@ static void IO_thrdata_free(thr_data_t *td)
 
 /*
  * Search data node for a FD
- * (This also frees already closed data)
+ *  - The FD is searched using a Key because using the FD itself may fail
+ *  due to a race condition between the FD close and its reuse.
+ *  - This function also frees already closed data.
  */
-static thr_data_t *IO_thrdata_find(gint FD)
+static thr_data_t *IO_thrdata_find(void *Key)
 {
    gint i, idx = -1;
 
+   _MSG("TL(%p): [", Key);
    for (i = 0; i < ThrDataSize; ++i) {
+      _MSG(" %d%s", ThrData[i]->FD, ThrData[i]->Done ? "D" : "");
       if (ThrData[i]->Done) {
          IO_thrdata_free(ThrData[i]);
          a_List_remove(ThrData, i, ThrDataSize);
          --i;
-      } else if (ThrData[i]->FD == FD) {
+      } else if (ThrData[i]->Key == Key) {
          idx = i;
+         _MSG("*");
       }
    }
+   _MSG("]\n");
 
    if (idx != -1)
       return ThrData[idx];
@@ -282,9 +292,12 @@ static void *IO_write_chunk(void *ptr)
       if (data->Buf2)
          continue;
       else if (data->LastChunk) {
-         do
-            st = close(data->FD);
-         while (st < 0 && errno == EINTR);
+         /* Only pipes are closed, sockets are left for the server to close */
+         if (data->CloseFD) {
+            do
+               st = close(data->FD);
+            while (st < 0 && errno == EINTR);
+         }
          data->Done = 1;
          pthread_mutex_unlock(&data->mut);
          _MSG("thr:: LastChunk:%d Done:%d  --Bailing out!\n",
@@ -301,33 +314,41 @@ static void *IO_write_chunk(void *ptr)
 
 /*
  * Write a data chunk from a pthread
- * (LastChunk -> close tansfer)
+ * (!Buf && !BufSize -> LastChunk -> close tansfer, don't close FD)
+ * (!Buf && BufSize == 1 -> LastChunk -> close tansfer, close FD)
  */
-void a_IO_write_chunk(gint FD, void *Buf, size_t BufSize)
+void a_IO_write_chunk(gint FD, void *Key, void *Buf, size_t BufSize)
 {
    thr_data_t *data;
+   gint new_thread = 0;
 
    /* Search data node for this FD */
-   data = IO_thrdata_find(FD);
+   data = IO_thrdata_find(Key);
+   _MSG(" a_IO_write_chunk: data=%p Buf=%p, BufSize=%d, FD=%d\n",
+        data, Buf, BufSize, FD);
 
-   if (!data) {
-      data = IO_thrdata_new(FD);
+   if (!data && Buf) {
+      data = IO_thrdata_new(FD, Key);
       a_List_add(ThrData, ThrDataSize, ThrDataMax);
       ThrData[ThrDataSize] = data;
       ThrDataSize++;
-      pthread_create(&data->thrID, NULL, IO_write_chunk, data);
+      new_thread = 1;
    }
 
    pthread_mutex_lock(&data->mut);
-    if (Buf && BufSize) {
+    if (Buf) {
        data->Buf2 = g_realloc(data->Buf2, data->Buf2Size + BufSize);
        memcpy(data->Buf2 + data->Buf2Size, Buf, BufSize);
        data->Buf2Size = data->Buf2Size + BufSize;
     } else {
        data->LastChunk = 1;
+       data->CloseFD = (BufSize == 1) ? 1 : 0;
     }
     pthread_cond_signal(&data->cond);
    pthread_mutex_unlock(&data->mut);
+
+   if (new_thread)
+      pthread_create(&data->thrID, NULL, IO_write_chunk, data);
 }
 
 
@@ -362,11 +383,11 @@ static IOData_t *IO_get(gint Key)
  */
 static void IO_free(IOData_t *io)
 {
-  g_return_if_fail(IO_get(io->Key) == NULL);
-
-  if (io->Flags & IOFlag_FreeIOBuf)
-     a_IO_set_buf(io, NULL, 0);
-  g_free(io);
+   g_return_if_fail(IO_get(io->Key) == NULL);
+ 
+   if (io->Flags & IOFlag_FreeIOBuf)
+      a_IO_set_buf(io, NULL, 0);
+   g_free(io);
 }
 
 /*
@@ -688,14 +709,18 @@ void a_IO_ccc(int Op, int Branch, int Dir, ChainLink *Info,
             break;
             }
          case OpEnd:
-            a_IO_write_chunk(*(int*)Info->LocalKey, NULL, 0);
+            _MSG(" Info=%p Info->LocalKey=%p FD=%d Data1=%p\n",
+                 Info, Info->LocalKey, *(int*)Info->LocalKey, Data1);
+            a_IO_write_chunk(*(int*)Info->LocalKey, Info,
+                             NULL, (Data1) ? 1 : 0);
             g_free(Info->LocalKey);
             break;
          case OpSend:
             {
             /* this part submits the data to the thread */
             DataBuf *dbuf = Data1;
-            a_IO_write_chunk(*(int*)Info->LocalKey, dbuf->Buf, dbuf->Size);
+            a_IO_write_chunk(*(int*)Info->LocalKey, Info,
+                             dbuf->Buf, dbuf->Size);
             break;
             }
          case OpAbort:
