@@ -254,7 +254,6 @@ static void Dpi_parse_token(conn_data_t *conn)
 
    tag = g_strndup(conn->Tok, conn->TokSize);
    MSG("Dpi_parse_token: [%s]\n", tag);
-   g_free(tag);
 
    cmd = Get_attr_value(conn->Tok, conn->TokSize, "cmd");
    if (strcmp(cmd, "send_status_message") == 0) {
@@ -267,6 +266,10 @@ static void Dpi_parse_token(conn_data_t *conn)
       a_Chain_fcb(OpSend, conn->InfoRecv, msg, cmd);
       g_free(msg);
 
+   } else if (strcmp(cmd, "dialog") == 0) {
+      /* For now will send the dpip tag... */
+      a_Chain_fcb(OpSend, conn->InfoRecv, tag, cmd);
+
    } else if (strcmp(cmd, "start_send_page") == 0) {
       urlstr = Get_attr_value(conn->Tok, conn->TokSize, "url");
       a_Chain_fcb(OpSend, conn->InfoRecv, urlstr, cmd);
@@ -280,6 +283,8 @@ static void Dpi_parse_token(conn_data_t *conn)
       g_free(urlstr);
    }
    g_free(cmd);
+
+   g_free(tag);
 }
 
 /*
@@ -374,7 +379,7 @@ gint Dpi_check_uds(gchar *uds_name)
       memset(&pun, 0, sizeof(struct sockaddr_un));
       pun.sun_family = AF_LOCAL;
       strncpy(pun.sun_path, uds_name, sizeof (pun.sun_path));
-   
+
       if ((SockFD = socket(AF_LOCAL, SOCK_STREAM, 0)) == -1 ||
           connect(SockFD, (void*)&pun, D_SUN_LEN(&pun)) == -1) {
          DEBUG_MSG(4, "Dpi_check_uds: %s %s\n", g_strerror(errno), uds_name);
@@ -396,7 +401,7 @@ gchar *Dpi_get_dpid_uds_dir(void)
    gchar *saved_name_filename;    /*  :)  */
    gchar dpid_uds_dir[256], *p;
 
-   saved_name_filename = 
+   saved_name_filename =
       g_strconcat(g_get_home_dir(), "/", ".dillo/dpi_socket_dir", NULL);
    in = fopen(saved_name_filename, "r");
    g_free(saved_name_filename);
@@ -499,7 +504,7 @@ gchar *Dpi_get_server_uds_name(gchar *server_name)
       dpid_uds_name = g_strconcat(dpid_uds_dir, "/", "dpid.srs", NULL);
       _MSG("dpid_uds_name = [%s]\n", dpid_uds_name);
       strncpy(dpid.sun_path, dpid_uds_name, sizeof(dpid.sun_path));
-      
+
       if (connect(sock, (struct sockaddr *) &dpid, sizeof(dpid)) == -1)
          perror("connect");
       /* ask dpid to check the server plugin and send its UDS name back */
@@ -778,7 +783,7 @@ void a_Dpi_bye_dpid()
    if (connect(new_socket, (struct sockaddr *) &sa, addr_len) == -1) {
       DEBUG_MSG(4, "a_Dpi_bye_dpid: %s\n", g_strerror(errno));
       fprintf(stderr, "%s\n", sa.sun_path);
-   } 
+   }
    (void) write(new_socket, DpiBye_cmd, strlen(DpiBye_cmd));
    close(new_socket);
 }
