@@ -29,18 +29,6 @@ static MimeItem_t *MimeMinItems = NULL;
 static gint MimeMajItemsSize = 0, MimeMajItemsMax = 8;
 static MimeItem_t *MimeMajItems = NULL;
 
-/*
- * NULL viewer (For unhandled content-types)
- */
-/*
-static DwWidget *Mime_null_viewer(const char *Type,
-                                  void *P,
-                                  CA_Callback_t *Call,
-                                  void **Data)
-{
-   return NULL;
-}
-*/
 
 /*
  * Add a specific MIME type (as "image/png") to our viewer list
@@ -131,7 +119,8 @@ void a_Mime_init()
  * Call the handler for the MIME type to set Call and Data as appropriate
  *
  * Return Value:
- *   A new Dw on success, otherwise NULL
+ *   On success: a new Dw (and Call and Data properly set).
+ *   On failure: NULL (and Call and Data untouched).
  */
 DwWidget *a_Mime_set_viewer(const char *content_type, void *Ptr,
                             CA_Callback_t *Call, void **Data)
@@ -160,6 +149,5 @@ DwWidget *a_Mime_set_viewer(const char *content_type, void *Ptr,
 
    /* Type not handled */
    MSG_HTTP("\n  Unhandled MIME type: <%s>\n", content_type);
-   *Call = a_Cache_null_client;
    return NULL;
 }
