@@ -11,7 +11,7 @@
 #include <gtk/gtk.h>
 #include "../dw_widget.h"
 #include "../cache.h"
-#include "../IO/Url.h"
+#include "Url.h"
 
 typedef DwWidget* (*Viewer_t) (const char*, void*, CA_Callback_t*, void**);
 
