@@ -446,32 +446,21 @@ gint a_Cache_open_url(void *web, CA_Callback_t Call, void *CbData)
  */
 gint a_Cache_get_buf(const DilloUrl *Url, gchar **PBuf, gint *BufSize)
 {
-   CacheData_t **stack = NULL;
-   gint stackSize = 0;
-   gint stackMax = 32;
    CacheData_t *entry;
-   gint i;
 
-   do {
-      entry = Cache_entry_search(Url);
+   while ((entry = Cache_entry_search(Url))) {
+
+      /* Test for a redirection loop */
+      if (entry->Flags & CA_RedirectLoop) {
+         g_warning ("Redirect loop for URL: >%s<\n", URL_STR_(Url));
+         break;
+      }
+      /* Test for a working redirection */
       if (entry && entry->Flags & CA_Redirect && entry->Location) {
-         a_List_add (stack, stackSize, stackMax);
-         stack[stackSize++] = entry;
          Url = entry->Location;
       } else
          break;
-
-      /* Test for a redirection loop */
-      for (i = 0; i < stackSize - 1; i++) {
-         if (stack[i] == entry) {
-            g_warning ("Redirect loop for URL: >%s<\n", URL_STR_(Url));
-            for (i = 0; i < stackSize; i++)
-               stack[i]->Flags |= CA_RedirectLoop;
-            entry = NULL;
-         }
-      }
-   } while (entry);
-   g_free (stack);
+   }
 
    *BufSize = (entry) ? entry->ValidSize : 0;
    *PBuf = (entry) ? (gchar *) entry->Data : NULL;
@@ -566,8 +555,10 @@ static void Cache_parse_header(CacheData_t *entry, IOData_t *io, gint HdrLen)
       /* 30x: URL redirection */
       entry->Flags |= CA_Redirect;
       if ( header[11] == '1' )
-         /* 301 Moved Permanently */
-         entry->Flags |= CA_ForceRedirect;
+         entry->Flags |= CA_ForceRedirect;  // 301 Moved Permanently
+      else if ( header[11] == '2' )
+         entry->Flags |= CA_TempRedirect;   // 302 Temporal Redirect
+     
       location_str = Cache_parse_field(header, "Location");
       entry->Location = a_Url_new(location_str, URL_STR_(entry->Url), 0, 0, 0);
       g_free(location_str);
@@ -642,7 +633,7 @@ static gint Cache_get_header(IOData_t *io, CacheData_t *entry)
 
    if ( N == 2 ){
       /* Got whole header */
-      DEBUG_MSG(2, "Header [io_len=%d]\n%s", i, hdr->str);
+      _MSG("Header [io_len=%d]\n%s", i, hdr->str);
       entry->Flags |= CA_GotHeader;
       /* Return number of original-header bytes in this io [1 based] */
       return i;
@@ -734,25 +725,32 @@ static gint Cache_redirect(CacheData_t *entry, gint Flags, BrowserWindow *bw)
 {
    DilloUrl *NewUrl;
 
+   _MSG(" Cache_redirect: redirect_level = %d\n", bw->redirect_level);
+
    /* if there's a redirect loop, stop now */
+   if (bw->redirect_level >= 5)
+      entry->Flags |= CA_RedirectLoop;
+
    if (entry->Flags & CA_RedirectLoop) {
      a_Interface_msg(bw, "ERROR: redirect loop for: %s", URL_STR_(entry->Url));
+     bw->redirect_level = 0;
      return 0;
    }
 
-   if ( ((entry->Flags & CA_Redirect) && entry->Location) &&
-        ((entry->Flags & CA_ForceRedirect) || !entry->ValidSize ||
-         entry->ValidSize < 1024 ) ) {
+   if ((entry->Flags & CA_Redirect && entry->Location) &&
+       (entry->Flags & CA_ForceRedirect || entry->Flags & CA_TempRedirect ||
+        !entry->ValidSize || entry->ValidSize < 1024 )) {
 
-      DEBUG_MSG(3, ">>>Redirect from: %s\n to %s\n",
-              URL_STR_(entry->Url), URL_STR_(entry->Location));
-      DEBUG_MSG(3,"%s", entry->Header->str);
+      _MSG(">>>Redirect from: %s\n to %s\n",
+           URL_STR_(entry->Url), URL_STR_(entry->Location));
+      _MSG("%s", entry->Header->str);
 
-      if ( Flags & WEB_RootUrl ) {
+      if (Flags & WEB_RootUrl) {
          /* Redirection of the main page */
-         a_Nav_remove_top_url(bw);
          NewUrl = a_Url_new(URL_STR_(entry->Location), URL_STR_(entry->Url),
                             0, 0, 0);
+         if (entry->Flags & CA_TempRedirect)
+            a_Url_set_flags(NewUrl, URL_FLAGS(NewUrl) | URL_E2EReload);
          a_Nav_push(bw, NewUrl);
          a_Url_free(NewUrl);
       } else {
@@ -768,12 +766,25 @@ static gint Cache_redirect(CacheData_t *entry, gint Flags, BrowserWindow *bw)
 }
 
 /*
- * Do nothing, but let the cache fill the entry.
- * (Currently used to ignore image redirects  --Jcid)
+ * Don't process data any further, but let the cache fill the entry.
+ * (Currently used to handle WEB_RootUrl redirects,
+ *  and to ignore image redirects --Jcid)
  */
 void a_Cache_null_client(int Op, CacheClient_t *Client)
 {
- return;
+   DilloWeb *Web = Client->Web;
+
+   /* make the stop button insensitive when done */
+   if (Op == CA_Close) {
+      if (Web->flags & WEB_RootUrl) {
+         /* Remove this client from our active list */
+         a_Interface_close_client(Web->bw, Client->Key);
+      }
+   }
+
+   /* else ignore */
+
+   return;
 }
 
 /*
@@ -814,6 +825,14 @@ static void Cache_process_queue(CacheData_t *entry)
                a_Interface_msg(Client_bw, "");
                entry->Flags |= CA_MsgErased;
             }
+            if (entry->Flags & CA_Redirect) {
+               if (!Client->Callback) {
+                  Client->Callback = a_Cache_null_client;
+                  Client_bw->redirect_level++;
+               }
+            } else {
+               Client_bw->redirect_level = 0;
+            }
          } else {
             /* For non root URLs, ignore redirections and 404 answers */
             if (entry->Flags & CA_Redirect || entry->Flags & CA_NotFound)
@@ -824,13 +843,18 @@ static void Cache_process_queue(CacheData_t *entry)
          if (!Client->Callback) {
             st = a_Web_dispatch_by_type(entry->Type, ClientWeb,
                                         &Client->Callback, &Client->CbData);
-            if (st == -1 && ClientWeb->flags & WEB_RootUrl) {
-               /* Unhandled MIME type, prepare a download offer... */
-               AbortEntry = TRUE;
-               a_Interface_remove_client(Client_bw, Client->Key);
-               Cache_client_dequeue(Client, NULLKey);
-               --i; /* Keep the index value in the next iteration */
-               continue;
+            if (st == -1) {
+               /* MIME type is not viewable */
+               Client->Callback = a_Cache_null_client;
+               Client->CbData = NULL;
+               if (ClientWeb->flags & WEB_RootUrl) {
+                  /* Unhandled MIME type, prepare a download offer... */
+                  AbortEntry = TRUE;
+                  a_Interface_remove_client(Client_bw, Client->Key);
+                  Cache_client_dequeue(Client, NULLKey);
+                  --i; /* Keep the index value in the next iteration */
+                  continue;
+               }
             }
          }
 
@@ -851,8 +875,13 @@ static void Cache_process_queue(CacheData_t *entry)
                (Client->Callback)(CA_Close, Client);
             Cache_client_dequeue(Client, NULLKey);
             --i; /* Keep the index value in the next iteration */
+
+            /* call Cache_redirect() from this 'if' to assert one call only. */
             if ( entry->Flags & CA_Redirect )
                Cache_redirect(entry, flags, Client_bw);
+
+            _MSG("Cache_process_queue: NumRootClients=%d sens_idle_id = %d\n",
+                 Client_bw->NumRootClients, Client_bw->sens_idle_id);
          }
       }
    } /* for */
@@ -941,7 +970,7 @@ static void Cache_stop_client(gint Key, gint force)
 
    if (!(List = g_slist_find_custom(ClientQueue, GINT_TO_POINTER(Key),
                                     Cache_client_key_cmp))){
-      DEBUG_MSG(5, "WARNING: Cache_stop_client, inexistent client\n");
+      _MSG("WARNING: Cache_stop_client, inexistent client\n");
       return;
    }
 
