@@ -350,7 +350,15 @@ static int Cookies_get_month(const char *month_name)
 
 /*
  * Return a local timestamp from a GMT date string
- * Accept: RFC-1123 | RFC-850 | ANSI asctime
+ * Accept: RFC-1123 | RFC-850 | ANSI asctime | Old Netscape format.
+ *
+ *   Wdy, DD-Mon-YY HH:MM:SS GMT
+ *   Wdy, DD-Mon-YYYY HH:MM:SS GMT
+ *   Weekday, DD-Mon-YY HH:MM:SS GMT
+ *   Weekday, DD-Mon-YYYY HH:MM:SS GMT
+ *   Tue May 21 13:46:22 1991\n
+ *   Tue May 21 13:46:22 1991
+ *
  * (return 0 on malformed date string syntax)
  */
 static time_t Cookies_create_timestamp(const char *expires)
@@ -361,9 +369,10 @@ static time_t Cookies_create_timestamp(const char *expires)
    gchar *E_msg =
       "Cookies: Expire date is malformed!\n"
       " (should be RFC-1123 | RFC-850 | ANSI asctime)\n"
-      " Ignoring cookie...\n";
+      " Ignoring cookie: [%s]\n";
 
-   if (!(cp = strchr(expires, ',')) && strlen(expires) == 24) {
+   cp = strchr(expires, ',');
+   if (!cp && (strlen(expires) == 24 || strlen(expires) == 25)) {
       /* Looks like ANSI asctime format... */
       cp = (gchar *)expires;
       day = strtol(cp + 8, NULL, 10);       /* day */
@@ -373,9 +382,9 @@ static time_t Cookies_create_timestamp(const char *expires)
       minutes = strtol(cp + 14, NULL, 10);  /* minutes */
       seconds = strtol(cp + 17, NULL, 10);  /* seconds */
 
-   } else if (cp && ((cp - expires == 3 && strlen(cp) == 26) ||
-                     (cp - expires > 5  && strlen(cp) == 24))) {
-      /* RFC-1123 | RFC-850 format */
+   } else if (cp && (cp - expires == 3 || cp - expires > 5) &&
+                    (strlen(cp) == 24 || strlen(cp) == 26)) {
+      /* RFC-1123 | RFC-850 format | Old Netscape format */
       day = strtol(cp + 2, NULL, 10);
       month = Cookies_get_month(cp + 5);
       year = strtol(cp + 9, &cp, 10);
@@ -386,15 +395,15 @@ static time_t Cookies_create_timestamp(const char *expires)
       seconds = strtol(cp + 7, NULL, 10);
 
    } else {
-      DEBUG_MSG(5, E_msg);
+      DEBUG_MSG(5, E_msg, expires);
       return (time_t) 0;
    }
 
    /* Error checks  --this may be overkill */
-   if (!(day > 0 && day < 32 && month > 0 && month && year > 1970 &&
+   if (!(day > 0 && day < 32 && month > 0 && month < 13 && year > 1970 &&
          hour >= 0 && hour < 24 && minutes >= 0 && minutes < 60 &&
          seconds >= 0 && seconds < 60)) {
-      DEBUG_MSG(5, E_msg);
+      DEBUG_MSG(5, E_msg, expires);
       return (time_t) 0;
    }
 
@@ -721,10 +730,8 @@ static CookieData_t *Cookies_parse_one(const DilloUrl *set_url,
          }
       } else if (g_strcasecmp(attr, "Expires") == 0) {
          if (!max_age && !discard) {
-            DEBUG_MSG(8,"Cookie has 'Expires' attribute "
-                        "which isn't RFC-2965 compliant.\n");
+            DEBUG_MSG(5, "Old netscape-style cookie...\n");
             value = Cookies_parse_value(&str, TRUE, FALSE);
-
             if (value) {
                cookie->expires_at = Cookies_create_timestamp(value);
                cookie->session_only = FALSE;
