@@ -10,7 +10,17 @@ extern "C" {
 
 void a_Dns_init (void);
 void a_Dns_freeall(void);
-void a_Dns_ccc(int Op, int Br, ChainLink *Info, void *Data, void *ExtraData);
+void a_Dns_ccc(int Op, int Dir, int Br, ChainLink *Info,
+               void *Data1, void *Data2);
+
+#define DILLO_ADDR_MAX 16
+
+typedef struct _DilloHost
+{
+  int af;
+  int alen;
+  char data[DILLO_ADDR_MAX];
+} DilloHost;
 
 #ifdef __cplusplus
 }
