@@ -1,7 +1,7 @@
 /*
  * File: dw_aligned_page.c
  *
- * Copyright (C) 2002  Sebastian Geerken <S.Geerken@ping.de>
+ * Copyright (C) 2002, 2003  Sebastian Geerken <S.Geerken@ping.de>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,6 +37,7 @@ static void   Dw_aligned_page_destroy            (GtkObject *object);
 
 static gint32 Dw_aligned_page_real_get_value     (DwAlignedPage *aligned_page);
 static void   Dw_aligned_page_real_set_max_value (DwAlignedPage *aligned_page,
+                                                  gint32 max_value,
                                                   gint32 value);
 
 static DwPageClass *parent_class;
@@ -96,7 +97,7 @@ static void Dw_aligned_page_destroy (GtkObject *object)
    DwAlignedPage *aligned_page = DW_ALIGNED_PAGE (object);
 
    if (aligned_page->list) {
-      if (aligned_page->list->num == 1) {
+      if (aligned_page->list->refcount == 1) {
          /* This is the last page in the array, so the list is removed. */
          g_free (aligned_page->list->pages);
          g_free (aligned_page->list->values);
@@ -106,10 +107,11 @@ static void Dw_aligned_page_destroy (GtkObject *object)
             NULL and 0. */
          aligned_page->list->pages[aligned_page->list_pos] = NULL;
          aligned_page->list->values[aligned_page->list_pos] = 0;
+         aligned_page->list->refcount--;
       }
    }
 
-   (* GTK_OBJECT_CLASS(parent_class)->destroy) (object);
+   GTK_OBJECT_CLASS(parent_class)->destroy (object);
 }
 
 
@@ -124,8 +126,9 @@ void p_Dw_aligned_page_set_ref_page (DwAlignedPage *aligned_page,
    DwAlignedPageList *list;
 
    if (ref_page == NULL) {
-      list = g_new(DwAlignedPageList, 1);
+      list = g_new (DwAlignedPageList, 1);
       list->num = 0;
+      list->refcount = 0;
       list->pages = NULL;
       list->values = NULL;
       list->num_pages_max = 4;
@@ -135,9 +138,10 @@ void p_Dw_aligned_page_set_ref_page (DwAlignedPage *aligned_page,
       list = ref_page->list;
 
    list->num++;
-   a_List_add(list->pages, list->num, list->num_pages_max);
+   list->refcount++;
+   a_List_add (list->pages, list->num, list->num_pages_max);
    list->pages[list->num - 1] = aligned_page;
-   a_List_add(list->values, list->num, list->num_values_max);
+   a_List_add (list->values, list->num, list->num_values_max);
    list->values[list->num - 1] = 0;
    aligned_page->list = list;
    aligned_page->list_pos = list->num - 1;
@@ -165,17 +169,20 @@ void p_Dw_aligned_page_update_value (DwAlignedPage *aligned_page)
          aligned_page->list->max_value =
             aligned_page->list->values[aligned_page->list_pos];
 
-         for(i = 0; i < aligned_page->list->num; i++)
+         for (i = 0; i < aligned_page->list->num; i++)
             if (aligned_page->list->pages[i]) {
                klass = DW_ALIGNED_PAGE_CLASS
                   (GTK_OBJECT(aligned_page->list->pages[i])->klass);
                klass->set_max_value (aligned_page->list->pages[i],
-                                     aligned_page->list->max_value);
+                                     aligned_page->list->max_value,
+                                     aligned_page->list->values[i]);
             }
       } else {
          /* No change, apply old max_value only to this page. */
          klass = DW_ALIGNED_PAGE_CLASS (GTK_OBJECT(aligned_page)->klass);
-         klass->set_max_value (aligned_page, aligned_page->list->max_value);
+         klass->set_max_value (aligned_page, aligned_page->list->max_value,
+                               aligned_page->list->values
+                                  [aligned_page->list_pos]);
       }
    }
 }
@@ -196,6 +203,7 @@ static gint32 Dw_aligned_page_real_get_value (DwAlignedPage *aligned_page)
  * Standard implementation of DwAlignedPage::set_max_value.
  */
 static void Dw_aligned_page_real_set_max_value (DwAlignedPage *aligned_page,
+                                                gint32 max_value,
                                                 gint32 value)
 {
    g_warning ("DwAlignedPage::set_max_value not implemented for `%s'",
