@@ -30,7 +30,7 @@ struct _DwButton
    DwContainer container;
 
    DwWidget *child;
-   gboolean in_button, pressed, sensitive;
+   gboolean relief, in_button, pressed, sensitive;
 };
 
 
@@ -38,12 +38,16 @@ struct _DwButtonClass
 {
    DwContainerClass parent_class;
 
-   void (*clicked)  (DwButton *button);
+   void (*clicked)    (DwButton *button);
+   void (*clicked_at) (DwButton *button,
+                       gint32 x,
+                       gint32 y);
 };
 
 
 GtkType    a_Dw_button_get_type      (void);
-DwWidget*  a_Dw_button_new           (void);
+DwWidget*  a_Dw_button_new           (gint flags,
+                                      gboolean relief);
 void       a_Dw_button_set_sensitive (DwButton *button,
                                       gboolean sensitive);
 
