@@ -153,7 +153,7 @@ static void Dw_gtk_scrolled_frame_init (GtkDwScrolledFrame *frame)
    frame->hadjustment = NULL;
    frame->vadjustment = NULL;
    frame->button2_pressed = FALSE;
-   frame->move_idle_id = -1;
+   frame->move_idle_id = 0;
 }
 
 
@@ -243,10 +243,10 @@ static void Dw_gtk_scrolled_frame_destroy (GtkObject *object)
    GtkDwScrolledFrame *frame;
 
    frame = GTK_DW_SCROLLED_FRAME (object);
-   if (frame->move_idle_id != -1)
+   if (frame->move_idle_id != 0)
       gtk_idle_remove (frame->move_idle_id);
 
-   (* GTK_OBJECT_CLASS(parent_class)->destroy) (object);
+   GTK_OBJECT_CLASS(parent_class)->destroy (object);
 }
 
 
@@ -415,7 +415,7 @@ static void Dw_gtk_scrolled_frame_draw_focus (GtkWidget *widget)
 static void Dw_gtk_scrolled_frame_draw (GtkWidget *widget,
                                         GdkRectangle *area)
 {
-   (* (GTK_WIDGET_CLASS(parent_class)->draw)) (widget, area);
+   GTK_WIDGET_CLASS(parent_class)->draw (widget, area);
    Dw_gtk_scrolled_frame_paint_shadow (widget, area);
 }
 
@@ -428,8 +428,7 @@ static gint Dw_gtk_scrolled_frame_expose (GtkWidget *widget,
 {
    gint ret_val;
 
-   ret_val =
-      (* (GTK_WIDGET_CLASS(parent_class)->expose_event)) (widget, event);
+   ret_val = GTK_WIDGET_CLASS(parent_class)->expose_event (widget, event);
    Dw_gtk_scrolled_frame_paint_shadow (widget, &(event->area));
 
    return ret_val;
@@ -450,23 +449,27 @@ static gint Dw_gtk_scrolled_frame_key_press (GtkWidget *widget,
 
    switch (event->keyval) {
    case GDK_Up:
-      Dw_gtk_scrolled_frame_move_by(frame, 0,
-                                   - frame->vadjustment->step_increment);
+   case GDK_KP_Up:
+      Dw_gtk_scrolled_frame_move_by (frame, 0,
+                                     - frame->vadjustment->step_increment);
       return TRUE;
 
    case GDK_Down:
-      Dw_gtk_scrolled_frame_move_by(frame, 0,
-                                   + frame->vadjustment->step_increment);
+   case GDK_KP_Down:
+      Dw_gtk_scrolled_frame_move_by (frame, 0,
+                                     + frame->vadjustment->step_increment);
       return TRUE;
 
    case GDK_Left:
-      Dw_gtk_scrolled_frame_move_by(frame,
-                                   - frame->hadjustment->step_increment, 0);
+   case GDK_KP_Left:
+      Dw_gtk_scrolled_frame_move_by (frame,
+                                     - frame->hadjustment->step_increment, 0);
       return TRUE;
 
    case GDK_Right:
-      Dw_gtk_scrolled_frame_move_by(frame,
-                                   + frame->hadjustment->step_increment, 0);
+   case GDK_KP_Right:
+      Dw_gtk_scrolled_frame_move_by (frame,
+                                     + frame->hadjustment->step_increment, 0);
       return TRUE;
 
    case GDK_Tab:
@@ -476,40 +479,46 @@ static gint Dw_gtk_scrolled_frame_key_press (GtkWidget *widget,
          return gtk_container_focus (container, GTK_DIR_TAB_FORWARD);
 
    case GDK_b: case GDK_B:
-      if (event->state & (GDK_MOD1_MASK | GDK_MOD2_MASK))
+      if (event->state & GDK_CONTROL_MASK)
          return FALSE;
       /* continues */
    case GDK_Page_Up:
+   case GDK_KP_Page_Up:
       if (event->state & GDK_CONTROL_MASK)
-         Dw_gtk_scrolled_frame_move_by(frame,
-                                      - frame->hadjustment->page_increment, 0);
+         Dw_gtk_scrolled_frame_move_by (frame,
+                                        - frame->hadjustment->page_increment,
+                                        0);
       else
-         Dw_gtk_scrolled_frame_move_by(frame, 0,
-                                      - frame->vadjustment->page_increment);
+         Dw_gtk_scrolled_frame_move_by (frame, 0,
+                                        - frame->vadjustment->page_increment);
       return TRUE;
 
    case GDK_Page_Down:
+   case GDK_KP_Page_Down:
    case GDK_space:
       if (event->state & GDK_CONTROL_MASK)
-         Dw_gtk_scrolled_frame_move_by(frame,
-                                      + frame->hadjustment->page_increment, 0);
+         Dw_gtk_scrolled_frame_move_by (frame,
+                                        + frame->hadjustment->page_increment,
+                                        0);
       else
-         Dw_gtk_scrolled_frame_move_by(frame, 0,
-                                      + frame->vadjustment->page_increment);
+         Dw_gtk_scrolled_frame_move_by (frame, 0,
+                                        + frame->vadjustment->page_increment);
       return TRUE;
 
    case GDK_Home:
+   case GDK_KP_Home:
       if (event->state & GDK_CONTROL_MASK)
-         Dw_gtk_scrolled_frame_move_x_to(frame, 0);
+         Dw_gtk_scrolled_frame_move_x_to (frame, 0);
       else
-         Dw_gtk_scrolled_frame_move_y_to(frame, 0);
+         Dw_gtk_scrolled_frame_move_y_to (frame, 0);
       return TRUE;
 
    case GDK_End:
+   case GDK_KP_End:
       if (event->state & GDK_CONTROL_MASK)
-         Dw_gtk_scrolled_frame_move_x_to(frame, G_MAXFLOAT);
+         Dw_gtk_scrolled_frame_move_x_to (frame, G_MAXFLOAT);
       else
-         Dw_gtk_scrolled_frame_move_y_to(frame, G_MAXFLOAT);
+         Dw_gtk_scrolled_frame_move_y_to (frame, G_MAXFLOAT);
       return TRUE;
 
    default:
@@ -619,10 +628,10 @@ static void Dw_gtk_scrolled_frame_add (GtkContainer *container,
 {
    GtkDwScrolledFrame *frame;
 
-   (* (GTK_CONTAINER_CLASS(parent_class)->add)) (container, widget);
+   GTK_CONTAINER_CLASS(parent_class)->add (container, widget);
 
    frame = GTK_DW_SCROLLED_FRAME (container);
-   if ( !gtk_widget_set_scroll_adjustments(
+   if ( !gtk_widget_set_scroll_adjustments (
            GTK_WIDGET (container), frame->hadjustment, frame->vadjustment) )
       g_warning ("Dw_gtk_scrolled_frame_set_scroll_adjustments(): "
                  "child is non scrollable");
@@ -643,26 +652,28 @@ static gint Dw_gtk_scrolled_frame_focus (GtkContainer *container,
 
    frame = GTK_DW_SCROLLED_FRAME (container);
 
+   if (!GTK_WIDGET_HAS_FOCUS (container) && container->focus_child == NULL){
+      // no focus on the GtkDwScrolledFrame or any child widget => grab focus!
+      gtk_widget_grab_focus (GTK_WIDGET (container));
+      return TRUE;
+   }
+
    switch (direction) {
    case GTK_DIR_TAB_FORWARD:
-   case GTK_DIR_RIGHT:
-   case GTK_DIR_DOWN:
-      if (container->focus_child == NULL && !GTK_WIDGET_HAS_FOCUS (container)){
-         /* no focus of the GtkDwScrolledFrame or a child widget -> focus */
+      /* deliver to child */
+      if (!gtk_container_focus (GTK_CONTAINER (GTK_BIN(container)->child),
+                                direction)) {
+         /* todo: it would be nice to keep focus on this last child widget,
+          * instead of the container... Anyway this may change with GTK2 */
          gtk_widget_grab_focus (GTK_WIDGET (container));
-         return TRUE;
-      } else
-         /* deliver to child */
-         return gtk_container_focus (GTK_CONTAINER (GTK_BIN(container)->child),
-                                     direction);
+      }
+      return TRUE;
 
    case GTK_DIR_TAB_BACKWARD:
-   case GTK_DIR_LEFT:
-   case GTK_DIR_UP:
-      if (GTK_WIDGET_HAS_FOCUS (container))
+      if (GTK_WIDGET_HAS_FOCUS (container)) {
          /* will focus the widget "before" */
          return FALSE;
-      else {
+      } else {
          if (!gtk_container_focus (GTK_CONTAINER (GTK_BIN(container)->child),
                                    GTK_DIR_TAB_BACKWARD))
             /* first child of the child unfocussed */
@@ -672,12 +683,19 @@ static gint Dw_gtk_scrolled_frame_focus (GtkContainer *container,
       }
       break;
 
+   /*
+   case GTK_DIR_LEFT:
+   case GTK_DIR_RIGHT:
+   case GTK_DIR_UP:
+   case GTK_DIR_DOWN:
+   */
    default:
-      /* make compiler happy (1) */
-      return FALSE;
+      /* focus the GtkDwScrolledFrame */
+      gtk_widget_grab_focus (GTK_WIDGET (container));
+      return TRUE;
    }
 
-   /* make compiler happy (2) */
+   /* make compiler happy */
    return FALSE;
 }
 
@@ -692,9 +710,9 @@ static gint Dw_gtk_scrolled_frame_focus (GtkContainer *container,
  * Standard Gtk+ function
  */
 static void
-Dw_gtk_scrolled_frame_set_scroll_adjustments(GtkDwScrolledFrame *frame,
-                                             GtkAdjustment *hadjustment,
-                                             GtkAdjustment *vadjustment)
+Dw_gtk_scrolled_frame_set_scroll_adjustments (GtkDwScrolledFrame *frame,
+                                              GtkAdjustment *hadjustment,
+                                              GtkAdjustment *vadjustment)
 {
    GtkBin *bin;
 
@@ -741,7 +759,7 @@ static gint Dw_gtk_scrolled_frame_move_idle (gpointer data)
       gtk_signal_emit (GTK_OBJECT (frame), frame_signals[USER_VCHANGED]);
    }
 
-   frame->move_idle_id = -1;
+   frame->move_idle_id = 0;
    return FALSE;
 }
 
@@ -754,7 +772,7 @@ static void Dw_gtk_scrolled_frame_move_x_to (GtkDwScrolledFrame *frame,
 {
    frame->moveto_x =
       Dw_gtk_scrolled_frame_correct_adj (frame->hadjustment, x);
-   if (frame->move_idle_id == -1)
+   if (frame->move_idle_id == 0)
       frame->move_idle_id = gtk_idle_add (Dw_gtk_scrolled_frame_move_idle,
                                           (gpointer) frame);
 }
@@ -768,7 +786,7 @@ static void Dw_gtk_scrolled_frame_move_y_to (GtkDwScrolledFrame *frame,
 {
    frame->moveto_y =
       Dw_gtk_scrolled_frame_correct_adj (frame->vadjustment, y);
-   if (frame->move_idle_id == -1)
+   if (frame->move_idle_id == 0)
       frame->move_idle_id = gtk_idle_add (Dw_gtk_scrolled_frame_move_idle,
                                           (gpointer) frame);
 }
@@ -781,7 +799,7 @@ static void Dw_gtk_scrolled_frame_move_by (GtkDwScrolledFrame *frame,
                                            gfloat dx,
                                            gfloat dy)
 {
-   if (frame->move_idle_id == -1) {
+   if (frame->move_idle_id == 0) {
       /*
        * Initialization: if the idle function is not active, set
        * moveto_x and moveto_y to adjustments values, which may
@@ -800,7 +818,7 @@ static void Dw_gtk_scrolled_frame_move_by (GtkDwScrolledFrame *frame,
       Dw_gtk_scrolled_frame_correct_adj (frame->vadjustment,
                                          frame->moveto_y + dy);
 
-   if (frame->move_idle_id == -1)
+   if (frame->move_idle_id == 0)
       frame->move_idle_id = gtk_idle_add (Dw_gtk_scrolled_frame_move_idle,
                                           (gpointer) frame);
 
