@@ -35,8 +35,9 @@ static void Dw_gtk_viewport_class_init    (GtkDwViewportClass *klass);
 static void Dw_gtk_viewport_destroy       (GtkObject *object);
 
 /* GtkWidget methods */
+static void Dw_gtk_viewport_size_allocate (GtkWidget *widget,
+                                           GtkAllocation *allocation);
 static void Dw_gtk_viewport_realize       (GtkWidget *widget);
-static void Dw_gtk_viewport_unrealize     (GtkWidget *widget);
 static void Dw_gtk_viewport_draw          (GtkWidget *widget,
                                            GdkRectangle *area);
 static gint Dw_gtk_viewport_expose        (GtkWidget *widget,
@@ -139,7 +140,7 @@ static void Dw_gtk_viewport_init (GtkDwViewport *viewport)
    viewport->draw_areas = NULL;
    viewport->num_draw_areas = 0;
    viewport->num_draw_areas_max = 4;
-   
+
    DBG_OBJ_ASSOC (viewport->findtext_state, viewport);
    DBG_OBJ_ASSOC (viewport->selection, viewport);
 }
@@ -160,8 +161,8 @@ static void Dw_gtk_viewport_class_init (GtkDwViewportClass *klass)
 
    object_class->destroy = Dw_gtk_viewport_destroy;
 
+   widget_class->size_allocate = Dw_gtk_viewport_size_allocate;
    widget_class->realize = Dw_gtk_viewport_realize;
-   widget_class->unrealize = Dw_gtk_viewport_unrealize;
    widget_class->draw = Dw_gtk_viewport_draw;
    widget_class->expose_event = Dw_gtk_viewport_expose;
    widget_class->button_press_event = Dw_gtk_viewport_button_press;
@@ -224,6 +225,19 @@ static void Dw_gtk_viewport_destroy (GtkObject *object)
  *                     *
  ***********************/
 
+/*
+ * Standard Gtk+ function
+ */
+static void Dw_gtk_viewport_size_allocate (GtkWidget *widget,
+                                           GtkAllocation *allocation)
+{
+   GTK_WIDGET_CLASS(parent_class)->size_allocate (widget, allocation);
+
+   /* gtk_layout_size_allocate() sets them to different values. */
+   GTK_LAYOUT(widget)->hadjustment->page_increment = allocation->width;
+   GTK_LAYOUT(widget)->vadjustment->page_increment = 0.96 * allocation->height;
+}
+
 /*
  * Standard Gtk+ function
  */
@@ -242,25 +256,12 @@ static void Dw_gtk_viewport_realize (GtkWidget *widget)
                           | GDK_LEAVE_NOTIFY_MASK);
 
    viewport = GTK_DW_VIEWPORT (widget);
-   if (viewport->child)
-      a_Dw_widget_realize (viewport->child);
    gdk_window_get_geometry (widget->window, NULL, NULL, NULL, NULL,
                             &viewport->depth);
-}
 
-
-/*
- * Standard Gtk+ function
- */
-static void Dw_gtk_viewport_unrealize (GtkWidget *widget)
-{
-   GtkDwViewport *viewport;
-
-   GTK_WIDGET_CLASS(parent_class)->unrealize (widget);
-
-   viewport = GTK_DW_VIEWPORT (widget);
+   Dw_gtk_viewport_update_background (viewport);
    if (viewport->child)
-      a_Dw_widget_unrealize (viewport->child);
+      Dw_widget_update_cursor (viewport->child);
 }
 
 
@@ -329,7 +330,7 @@ static void Dw_gtk_viewport_paint (GtkWidget *widget,
                                 TRUE, area->x, area->y,
                                 area->width, area->height);
             /* Widgets draw in backing pixmap. */
-            a_Dw_widget_draw (viewport->child, &intersection, event);
+            p_Dw_widget_draw (viewport->child, &intersection, event);
             /* Copy backing pixmap into window. */
             gdk_draw_pixmap (layout->bin_window, widget->style->black_gc,
                              viewport->back_pixmap, area->x, area->y,
@@ -468,6 +469,21 @@ static void Dw_gtk_viewport_adj_changed (GtkAdjustment *adj,
                                 viewport->mouse_x, viewport->mouse_y, NULL);
 }
 
+/*
+ * This function sets the background and the viewport.
+ */
+void Dw_gtk_viewport_update_background (GtkDwViewport *viewport)
+{
+   /* The toplevel widget should always have a defined background color,
+    * except at the beginning. Searching a defined background is not
+    * necessary. */
+   if (viewport->child && viewport->child->style && 
+       viewport->child->style->background_color)
+      gdk_window_set_background (GTK_LAYOUT(viewport)->bin_window,
+                                 &viewport->child->style->background_color
+                                 ->color);
+}
+
 /**********************
  *                    *
  *  public functions  *
@@ -489,10 +505,9 @@ void a_Dw_gtk_viewport_add_dw (GtkDwViewport *viewport,
 
    widget->parent = NULL;
    widget->viewport = GTK_WIDGET (viewport);
-   /*widget->window = viewport->back_pixmap;*/
 
-   if (GTK_WIDGET_REALIZED (viewport))
-      a_Dw_widget_realize (widget);
+   Dw_gtk_viewport_update_background (viewport);
+   Dw_widget_update_cursor (viewport->child);
 
    Dw_gtk_viewport_calc_size (viewport);
    Dw_gtk_viewport_remove_anchor (viewport);
@@ -535,11 +550,11 @@ static void Dw_gtk_viewport_calc_child_size (GtkDwViewport *viewport,
    DEBUG_MSG (2, "   width = %d, height = %d ...\n",
               child_width, child_height);
 
-   a_Dw_widget_set_width (viewport->child, child_width);
-   a_Dw_widget_set_ascent (viewport->child, child_height);
-   a_Dw_widget_set_descent (viewport->child, 0);
+   p_Dw_widget_set_width (viewport->child, child_width);
+   p_Dw_widget_set_ascent (viewport->child, child_height);
+   p_Dw_widget_set_descent (viewport->child, 0);
 
-   a_Dw_widget_size_request (viewport->child, child_requisition);
+   p_Dw_widget_size_request (viewport->child, child_requisition);
 }
 
 
@@ -672,7 +687,7 @@ void Dw_gtk_viewport_calc_size (GtkDwViewport *viewport)
       child_allocation.width = child_requisition.width;
       child_allocation.ascent = child_requisition.ascent;
       child_allocation.descent = child_requisition.descent;
-      a_Dw_widget_size_allocate (viewport->child, &child_allocation);
+      p_Dw_widget_size_allocate (viewport->child, &child_allocation);
 
       gtk_layout_set_size (GTK_LAYOUT (viewport),
                            child_requisition.width + 2 * border_width,
@@ -766,7 +781,7 @@ DwWidget* Dw_gtk_viewport_widget_at_point (GtkDwViewport *viewport,
 /*
  * Add or change an anchor.
  * The widget is responsible for storing a copy of name.
- * 
+ *
  * todo: Currently, no horizontal scrolling is done. This is generally
  * possible, DW_HPOS_INTO_VIEW should be used for this, but it is
  * rather complicated to determine the width of an anchor. This would
@@ -1063,7 +1078,7 @@ static gint Dw_gtk_viewport_draw_resize_idle (gpointer data)
          viewport_area.y = gtk_layout_get_vadjustment(layout)->value;;
          viewport_area.width = widget->allocation.width;
          viewport_area.height = widget->allocation.height;
-      
+
          if (p_Dw_rectangle_intersect (&viewport->draw_areas[i],
                                        &viewport_area, &world_area)) {
             gtk_area.x = world_area.x - viewport_area.x;
@@ -1073,16 +1088,16 @@ static gint Dw_gtk_viewport_draw_resize_idle (gpointer data)
             gtk_widget_draw (widget, &gtk_area);
          }
       }
-      
+
       /* No more areas to be drawn. */
       viewport->num_draw_areas = 0;
       break;
-      
+
    case DW_GTK_VIEWPORT_RESIZE:
       Dw_gtk_viewport_calc_size (viewport);
       break;
    }
-   
+
    viewport->draw_resize_idle_id = 0;
    return FALSE;
 }
@@ -1154,7 +1169,7 @@ void Dw_gtk_viewport_queue_resize (GtkDwViewport *viewport)
    /* Resizing always overrides drawing. */
    viewport->draw_resize_action = DW_GTK_VIEWPORT_RESIZE;
    viewport->num_draw_areas = 0;
-   
+
    if (viewport->draw_resize_idle_id == 0)
       viewport->draw_resize_idle_id =
          gtk_idle_add (Dw_gtk_viewport_draw_resize_idle, (gpointer)viewport);
@@ -1175,5 +1190,5 @@ DwWidget* a_Dw_gtk_viewport_widget_at_viewport_point (GtkDwViewport *viewport,
       world_y = vy + gtk_layout_get_vadjustment(GTK_LAYOUT(viewport))->value;
       return Dw_gtk_viewport_widget_at_point (viewport, world_x, world_y);
    } else
-      return NULL;   
+      return NULL;
 }
