@@ -414,7 +414,7 @@ static void Dw_page_get_word_extremes (DwPageWord *word,
 {
    if (word->content.type == DW_CONTENT_WIDGET) {
       if (DW_WIDGET_USES_HINTS (word->content.data.widget))
-         a_Dw_widget_get_extremes (word->content.data.widget, extremes);
+         p_Dw_widget_get_extremes (word->content.data.widget, extremes);
       else {
          if (DW_STYLE_IS_PER_LENGTH(word->content.data.widget->style->width)) {
             extremes->min_width = 0;
@@ -430,7 +430,7 @@ static void Dw_page_get_word_extremes (DwPageWord *word,
                DW_STYLE_ABS_LENGTH_VAL(word->content.data.widget->style->width)
                + p_Dw_style_box_diff_width(word->style);
          } else
-            a_Dw_widget_get_extremes (word->content.data.widget, extremes);
+            p_Dw_widget_get_extremes (word->content.data.widget, extremes);
       }
    } else {
       extremes->min_width = word->size.width;
@@ -463,7 +463,7 @@ static void Dw_page_get_extremes (DwWidget *widget,
       /* no rewrap necessary -> values in lines are up to date */
       line = &page->lines[page->num_lines - 1];
       /* Historical note: The former distinction between lines with and without
-       * words[first_word]->nowrap set is no longer necessary, since 
+       * words[first_word]->nowrap set is no longer necessary, since
        * Dw_page_real_word_wrap sets max_word_min to the correct value in any
        * case. */
       extremes->min_width = line->max_word_min;
@@ -621,7 +621,7 @@ static void Dw_page_size_allocate (DwWidget *widget,
                word->content.data.widget->style->margin.top;
             child_allocation.descent = word->size.descent +
                word->content.data.widget->style->margin.bottom;
-            a_Dw_widget_size_allocate (word->content.data.widget,
+            p_Dw_widget_size_allocate (word->content.data.widget,
                                        &child_allocation);
             break;
 
@@ -799,22 +799,22 @@ static void Dw_page_justify_line (DwPage *page, DwPageLine *line,
       orig_space_sum = 0;
       for (i = line->first_word; i < line->last_word - 1; i++)
          orig_space_sum += page->words[i].orig_space;
-      
+
       orig_space_cum = 0;
       last_eff_space_diff_cum = 0;
       for (i = line->first_word; i < line->last_word - 1; i++) {
          orig_space_cum += page->words[i].orig_space;
-         
+
          if (orig_space_cum == 0)
             eff_space_diff_cum = last_eff_space_diff_cum;
          else
             eff_space_diff_cum = diff * orig_space_cum / orig_space_sum;
-         
+
          page->words[i].eff_space = page->words[i].orig_space +
             (eff_space_diff_cum - last_eff_space_diff_cum);
          DBG_OBJ_ARRSET_NUM (page, "words.%d.eff_space", i,
                              page->words[i].eff_space);
-         
+
          last_eff_space_diff_cum = eff_space_diff_cum;
       }
    }
@@ -836,14 +836,14 @@ static void Dw_page_add_line (DwPage *page, gint word_ind, gboolean new_par)
 
    DEBUG_MSG (DEBUG_REWRAP_LEVEL, "--- new line %d in %p, with word %d of %d"
               "\n", page->num_lines - 1, page, word_ind, page->num_words);
-   
+
    last_line = &page->lines[page->num_lines - 1];
-   
+
    if (page->num_lines == 1)
       plast_line = NULL;
    else
       plast_line = &page->lines[page->num_lines - 2];
-   
+
    if (plast_line) {
       /* second or more lines: copy values of last line */
       last_line->top =
@@ -876,14 +876,14 @@ static void Dw_page_add_line (DwPage *page, gint word_ind, gboolean new_par)
                        last_line->par_min);
    DBG_OBJ_ARRSET_NUM (page, "lines.%d.par_max", page->num_lines - 1,
                        last_line->par_max);
-   
+
    last_line->first_word = word_ind;
    last_line->ascent = 0;
    last_line->descent = 0;
    last_line->margin_descent = 0;
    last_line->break_space = 0;
    last_line->left_offset = 0;
-   
+
    DBG_OBJ_ARRSET_NUM (page, "lines.%d.ascent", page->num_lines - 1,
                        last_line->ascent);
    DBG_OBJ_ARRSET_NUM (page, "lines.%d.descent", page->num_lines - 1,
@@ -892,7 +892,7 @@ static void Dw_page_add_line (DwPage *page, gint word_ind, gboolean new_par)
    /* update values in line */
    last_line->max_line_width =
       MAX (last_line->max_line_width, page->last_line_width);
-   
+
    if (page->num_lines > 1)
       page->last_line_width = 0;
    else
@@ -1030,13 +1030,13 @@ static void Dw_page_real_word_wrap (DwPage *page, gint word_ind)
 
    if (word->content.type == DW_CONTENT_WIDGET) {
       last_line->margin_descent =
-         MAX (last_line->margin_descent, 
+         MAX (last_line->margin_descent,
               word->size.descent +
               word->content.data.widget->style->margin.bottom);
 
       DBG_OBJ_ARRSET_NUM (page, "lines.%d.descent", page->num_lines - 1,
                           last_line->descent);
-      
+
       /* If the widget is not in the first line of the paragraph, its top
        * margin may make the line higher.
        */
@@ -1160,10 +1160,10 @@ static void Dw_page_calc_widget_size (DwPage *page,
    avail_descent = page->avail_descent;
 
    if (DW_WIDGET_USES_HINTS (widget)) {
-      a_Dw_widget_set_width (widget, avail_width);
-      a_Dw_widget_set_ascent (widget, avail_ascent);
-      a_Dw_widget_set_descent (widget, avail_descent);
-      a_Dw_widget_size_request (widget, size);
+      p_Dw_widget_set_width (widget, avail_width);
+      p_Dw_widget_set_ascent (widget, avail_ascent);
+      p_Dw_widget_set_descent (widget, avail_descent);
+      p_Dw_widget_size_request (widget, size);
       size->ascent -= widget->style->margin.top;
       size->descent -= widget->style->margin.bottom;
    } else {
@@ -1171,7 +1171,7 @@ static void Dw_page_calc_widget_size (DwPage *page,
        * (No harm for the next future.) */
       if (widget->style->width == DW_STYLE_LENGTH_AUTO ||
           widget->style->height == DW_STYLE_LENGTH_AUTO)
-         a_Dw_widget_size_request (widget, &requisition);
+         p_Dw_widget_size_request (widget, &requisition);
 
       if (widget->style->width == DW_STYLE_LENGTH_AUTO)
          size->width = requisition.width;
@@ -1343,7 +1343,7 @@ static void Dw_page_draw_line (DwPage *page,
       diff = 0;
       gc = word->style->color->gc;
 
-      
+
       DBG_OBJ_ARRSET_NUM (page, "words.%d.<i>drawn at</i>.x", word_index,
                           x_widget);
       DBG_OBJ_ARRSET_NUM (page, "words.%d.<i>drawn at</i>.y", word_index,
@@ -1487,7 +1487,7 @@ static void Dw_page_draw_line (DwPage *page,
       case DW_CONTENT_WIDGET:
          child = word->content.data.widget;
          if (p_Dw_widget_intersect (child, area, &child_area))
-            a_Dw_widget_draw (child, &child_area, event);
+            p_Dw_widget_draw (child, &child_area, event);
          break;
 
       case DW_CONTENT_ANCHOR: case DW_CONTENT_BREAK:
@@ -1991,7 +1991,7 @@ void a_Dw_page_add_text (DwPage *page, char *text, DwStyle *style)
                             style);
    word->content.type = DW_CONTENT_TEXT;
    word->content.data.text = text;
-   
+
    DBG_OBJ_ARRSET_STR (page, "words.%d.content.text", page->num_words - 1,
                        word->content.data.text);
 
@@ -2073,6 +2073,10 @@ void a_Dw_page_add_space (DwPage *page, DwStyle *style)
    if (nl >= 0) {
       nw = page->num_words - 1;
       if (nw >= 0) {
+         /* todo: remove this test case */
+         if (page->words[nw].orig_space != 0) {
+            _MSG("   a_Dw_page_add_space:: already existing space!!!\n");
+         }
          space = style->font->space_width;
          page->words[nw].orig_space = space;
          page->words[nw].eff_space = space;
@@ -2283,7 +2287,7 @@ void a_Dw_page_change_word_style (DwPage *page, gint from, gint to,
             old_style = word->style;
             word->style = style;
             a_Dw_style_ref (style);
-            
+
             if ((diffs = a_Dw_style_size_diffs (old_style, style))) {
                must_queue_resize = TRUE;
                break_line = MIN (break_line, line);
@@ -2331,7 +2335,7 @@ void a_Dw_page_change_word_style (DwPage *page, gint from, gint to,
             (widget, 0, Dw_page_line_total_y_offset_i (page, line),
              widget->allocation.width,
              page->lines[line].ascent + page->lines[line].descent);
-         
+
          if (i < to - 1)
             line = Dw_page_find_line_of_word (page, i + 1);
       }
@@ -2509,12 +2513,12 @@ static void Dw_page_iterator_get_allocation (DwIterator *it,
       else
          allocation->width = 0;
    }
-   
+
    allocation->width -= diff;
 
    allocation->ascent = page->words[wi].size.ascent;
    allocation->descent = page->words[wi].size.descent;
-   
+
    DBG_MSGF (page, "scrolling", 0,
              "Dw_page_iterator_get_allocation: (%d, %d), (%d x %d x %d)",
              allocation->x, allocation->y,
