@@ -19,7 +19,6 @@ extern "C" {
 #define DW_IS_PAGE(obj)         GTK_CHECK_TYPE ((obj), DW_TYPE_PAGE)
 #define DW_IS_PAGE_CLASS(klass) GTK_CHECK_CLASS_TYPE ((klass), DW_TYPE_PAGE)
 
-
 typedef struct _DwPage        DwPage;
 typedef struct _DwPageClass   DwPageClass;
 
@@ -27,30 +26,32 @@ typedef struct _DwPageClass   DwPageClass;
 typedef struct _DwPageLine   DwPageLine;
 typedef struct _DwPageWord   DwPageWord;
 
-
 struct _DwPageLine
 {
    gint first_word;    /* first-word's position in DwPageWord [0 based] */
    gint last_word;     /* last-word's position in DwPageWord [1 based] */
 
+   /* "top" is always relative to the top of the first line, i.e.
+    * page->lines[0].top is always 0. */
    gint32 top, ascent, descent, break_space, left_offset;
 
+   /* This is similar to descent, but includes the bottom margins of the
+    * widgets within this line. */
+   gint32 margin_descent;
+
    /* The following members contain accumulated values, from the top
     * down to the line before. */
    gint32 max_line_width; /* maximum of all line widths */
    gint32 max_word_min;   /* maximum of all word minima */
    gint32 max_par_max;    /* maximum of all paragraph maxima */
    gint32 par_min;        /* the minimal total width down from the last
-                             paragraph start */
+                             paragraph start, to the *beginning* of the
+                             line */
    gint32 par_max;        /* the maximal total width down from the last
-                             paragraph start */
+                             paragraph start, to the *beginning* of the
+                             line */
 };
 
-#define DW_PAGE_CONTENT_TEXT   0
-#define DW_PAGE_CONTENT_WIDGET 1
-#define DW_PAGE_CONTENT_ANCHOR 2
-#define DW_PAGE_CONTENT_BREAK  3
-
 struct _DwPageWord {
    /* todo: perhaps add a x_left? */
    DwRequisition size;
@@ -58,16 +59,9 @@ struct _DwPageWord {
    gint32 orig_space; /* from font, set by a_Dw_page_add_space */
    gint32 eff_space;  /* effective space, set by Dw_page_word_wrap,
                          used for drawing etc. */
+   gint hl_start[DW_HIGHLIGHT_NUM_LAYERS], hl_end[DW_HIGHLIGHT_NUM_LAYERS];
 
-   /* This is a variant record (i.e. it could point to a widget
-    * instead of just being text). */
-   gint content_type;
-   union {
-      char *text;
-      DwWidget *widget;
-      char *anchor;
-      gint break_space;
-   } content;
+   DwContent content;
 
    DwStyle *style;
    DwStyle *space_style; /* initially the same as of the word, later
@@ -80,13 +74,31 @@ struct _DwPage
    DwContainer container;
 
    /* These fields provide some ad-hoc-functionality, used by sub-classes. */
-   gboolean list_item;    /* If TRUE, the first word of the page is treated
-                             specially (search in source). */
-   gint32 inner_padding;  /* This is an additional padding on the left side
-                             (used by DwListItem). */
-   gint32 line1_offset;   /* This is an additional offset of the first line.
-                             May be negative (shift to left) or positive
-                             (shift to right). */
+   gboolean list_item;      /* If TRUE, the first word of the page is treated
+                               specially (search in source). */
+   gint32 inner_padding;    /* This is an additional padding on the left side
+                               (used by DwListItem). */
+   gint32 line1_offset;     /* This is an additional offset of the first line.
+                               May be negative (shift to left) or positive
+                               (shift to right). */
+   gint32 line1_offset_eff; /* The "effective" value of line1_offset, may
+                               differ from line1_offset when
+                               ignore_line1_offset_sometimes is set to TRUE */
+
+   /* The following is really hackish: It is used for DwTableCell (see
+    * comment in dw_table_cell.c), to avoid too wide table columns. If
+    * set to TRUE, it has following effects:
+    *
+    *  (i) line1_offset is ignored in calculating the minimal width
+    *      (which is used by DwTable!), and
+    * (ii) line1_offset is ignored (line1_offset_eff is set to 0),
+    *      when line1_offset plus the width of the first word is
+    *      greater than the the available witdh.
+    *
+    * todo: Eliminate all these ad-hoc features by a new, simpler and
+    *       more elegant design. ;-)
+    */
+   gboolean ignore_line1_offset_sometimes;
 
    gboolean must_queue_resize;
 
@@ -94,7 +106,9 @@ struct _DwPage
    gint32 avail_width, avail_ascent, avail_descent;
 
    gint32 last_line_width;
-   gint wrap_ref;
+   gint32 last_line_par_min;
+   gint32 last_line_par_max;
+   gint wrap_ref;  /* [0 based] */
 
    DwPageLine *lines;
    gint num_lines;
@@ -104,11 +118,13 @@ struct _DwPage
    gint num_words;
    gint num_words_max; /* number allocated */
 
-   /* The link under a button press */
-   gint link_pressed;
+   /* The word index of the link under a button press, and the char
+    * position */
+   gint link_pressed_index;
+   gint link_pressed_char_pos;
 
-   /* The link under the button */
-   gint hover_link;
+   gint hover_link;          /* The link under the button. */
+   DwTooltip *hover_tooltip; /* The tooltip under the button. No ref hold. */
 };
 
 
@@ -116,17 +132,20 @@ struct _DwPageClass
 {
    DwContainerClass parent_class;
 
-   void (*link_entered)  (DwPage *page,
-                          gint link, gint x, gint y);
-   void (*link_pressed)  (DwPage *page,
-                          gint link, gint x, gint y,
-                          GdkEventButton *event);
-   void (*link_released) (DwPage *page,
-                          gint link, gint x, gint y,
-                          GdkEventButton *event);
-   void (*link_clicked)  (DwPage *page,
-                          gint link, gint x, gint y,
-                          GdkEventButton *event);
+   gboolean (*link_entered)  (DwPage *page,
+                              gint link, gint x, gint y);
+   gboolean (*link_pressed)  (DwPage *page,
+                              gint link, gint x, gint y,
+                              GdkEventButton *event);
+   gboolean (*link_released) (DwPage *page,
+                              gint link, gint x, gint y,
+                              GdkEventButton *event);
+   gboolean (*link_clicked)  (DwPage *page,
+                              gint link, gint x, gint y,
+                              GdkEventButton *event);
+
+   void     (*word_wrap)     (DwPage *page,
+                              gint word_ind);
 };
 
 
@@ -134,6 +153,7 @@ DwWidget*  a_Dw_page_new           (void);
 GtkType    a_Dw_page_get_type      (void);
 
 void a_Dw_page_flush (DwPage *page);
+void a_Dw_page_set_voffset (DwPage *page, gint32 voffset);
 void a_Dw_page_add_text (DwPage *page, char *text, DwStyle *style);
 void a_Dw_page_add_widget (DwPage *page,
                            DwWidget *widget,
@@ -143,6 +163,11 @@ void a_Dw_page_add_space(DwPage *page, DwStyle *style);
 void a_Dw_page_add_parbreak (DwPage *page, gint space, DwStyle *style);
 void a_Dw_page_add_linebreak (DwPage *page, DwStyle *style);
 void a_Dw_page_hand_over_break (DwPage *page, DwStyle *style);
+void a_Dw_page_change_link_color (DwPage *page, gint link, gint32 new_color);
+void a_Dw_page_change_word_style (DwPage *page, gint from, gint to,
+                                  DwStyle *style, gboolean include_first_space,
+                                  gboolean include_last_space);
+
 
 #ifdef __cplusplus
 }
