@@ -16,9 +16,9 @@
 #include "prefs.h"
 #include "dw_gtk_viewport.h"
 
-#define DEBUG_CALC_LEVEL      0
+#define DEBUG_CALC_LEVEL     10
 #define DEBUG_EXTR_LEVEL     10
-#define DEBUG_WIDTH_LEVEL     0
+#define DEBUG_WIDTH_LEVEL    10
 #define DEBUG_ITERATOR_LEVEL  0
 
 /*#define DEBUG_LEVEL 10*/
@@ -278,7 +278,7 @@ static void Dw_table_size_request (DwWidget *widget,
 
    /* Calculate the widths of the columns, and store it in table->col_width. */
    if (table->num_cols > 0) {
-      a_Dw_widget_get_extremes (DW_WIDGET (table), &extremes);
+      p_Dw_widget_get_extremes (DW_WIDGET (table), &extremes);
 
       if (DW_STYLE_IS_ABS_LENGTH (widget->style->width))
          avail_width = DW_STYLE_ABS_LENGTH_VAL (widget->style->width);
@@ -335,7 +335,7 @@ static void Dw_table_size_request (DwWidget *widget,
          if (CHILD_DEFINED (table, n)) {
             child = table->children[n]->data.cell.widget;
             if (child->style->valign == DW_STYLE_VALIGN_BASELINE) {
-               a_Dw_widget_size_request (child, &child_requisition);
+               p_Dw_widget_size_request (child, &child_requisition);
                if (child_requisition.ascent > table->baseline[row])
                   table->baseline[row] = child_requisition.ascent;
             }
@@ -353,11 +353,11 @@ static void Dw_table_size_request (DwWidget *widget,
                  col2++)
                set_width += table->col_width[col2];
 
-            a_Dw_widget_set_width (child, set_width);
-            a_Dw_widget_set_ascent (child, table->avail_ascent);
-            a_Dw_widget_set_descent (child, table->avail_descent);
+            p_Dw_widget_set_width (child, set_width);
+            p_Dw_widget_set_ascent (child, table->avail_ascent);
+            p_Dw_widget_set_descent (child, table->avail_descent);
 
-            a_Dw_widget_size_request (child, &child_requisition);
+            p_Dw_widget_size_request (child, &child_requisition);
             /* Cells with valign=baseline have a height which may
              * be greater than the height returned by  size_request. */
             if (child->style->valign == DW_STYLE_VALIGN_BASELINE)
@@ -477,7 +477,7 @@ static void Dw_table_size_allocate (DwWidget *widget,
             child_allocation.width = cell_width;
 
             /* Vertical alignment is done via allocation. */
-            a_Dw_widget_size_request (child, &child_requisition);
+            p_Dw_widget_size_request (child, &child_requisition);
             switch (child->style->valign) {
             case DW_STYLE_VALIGN_TOP:
                child_allocation.ascent = child_requisition.ascent;
@@ -508,7 +508,7 @@ static void Dw_table_size_allocate (DwWidget *widget,
                break;
             }
 
-            a_Dw_widget_size_allocate (child, &child_allocation);
+            p_Dw_widget_size_allocate (child, &child_allocation);
          }
 
          col_offset += table->col_width[col] + widget->style->border_spacing;
@@ -608,7 +608,7 @@ static void Dw_table_draw (DwWidget *widget,
                 child->allocation.width + 2,
                 DW_WIDGET_HEIGHT(child) + 2);
 #endif
-            a_Dw_widget_draw (child, &child_area, event);
+            p_Dw_widget_draw (child, &child_area, event);
          }
       }
 
@@ -1233,7 +1233,7 @@ static void Dw_table_sub_get_extremes (DwTableSub *sub)
 
             if (row == start_row) {
                child = sub->table->children[start_n]->data.cell.widget;
-               a_Dw_widget_get_extremes (child, &child_extremes);
+               p_Dw_widget_get_extremes (child, &child_extremes);
                colspan = sub->table->children[start_n]->data.cell.colspan;
 
                /* Adjust width argument of the cell */
@@ -1432,7 +1432,7 @@ static void Dw_table_sub_calc_col_widths (DwTableSub *sub,
 {
    enum { AT_MIN, AT_MAX, AT_NORM }; /* for sub_status */
    gint *sub_status;
-   gint32 avail_width, sub_width, width_norm_cols;
+   gint32 avail_width, sub_width, width_norm_cols, diff_spacing;
    gint32 col_width, sum_sub_min_widths, sum_orig_sub_min_widths;
    gint i, num_cols, cols_per_sub, num_norm_cols, num_nf_subs, num_nf_cols;
    gint rest_w, rest_n, delta;
@@ -1529,7 +1529,8 @@ static void Dw_table_sub_calc_col_widths (DwTableSub *sub,
             DEBUG_MSG (DEBUG_WIDTH_LEVEL + 2, "%d\n", sub_width);
          }
       } else {
-         /* ---- STEP 3: Eventually, increase the sub-subtable extremes ---- */
+         /* ---- STEP 3: If necessary, increase the sub-subtable ----
+          * ---- extremes.                                       ---- */
          num_nf_subs = 0;
          num_nf_cols = 0;
          sum_nf_sub_extremes.min_width = 0;
@@ -1550,12 +1551,16 @@ static void Dw_table_sub_calc_col_widths (DwTableSub *sub,
             num_nf_cols = num_cols;
          }
 
+         diff_spacing = 
+            (sub->end_col - sub->start_col - 1)
+            * DW_WIDGET(sub->table)->style->border_spacing;
+
          Dw_table_sub_adjust_col_widths
-            (sub, FALSE, extremes.min_width, sub_extremes, num_nf_subs,
-             num_nf_cols, sum_nf_sub_extremes.min_width);
+            (sub, FALSE, extremes.min_width - diff_spacing, sub_extremes,
+             num_nf_subs, num_nf_cols, sum_nf_sub_extremes.min_width);
          Dw_table_sub_adjust_col_widths
-            (sub, TRUE, extremes.max_width, sub_extremes, num_nf_subs,
-             num_nf_cols, sum_nf_sub_extremes.max_width);
+            (sub, TRUE, extremes.max_width - diff_spacing, sub_extremes,
+             num_nf_subs, num_nf_cols, sum_nf_sub_extremes.max_width);
 
          /* ---- STEP 4: Finally, calculate the widths. ---- */
          sub_status = g_new (gint, sub->num_subs);
