@@ -36,20 +36,8 @@ static void Dw_widget_real_size_request   (DwWidget *widget,
 static void Dw_widget_real_get_extremes   (DwWidget *widget,
                                            DwExtremes *extremes);
 
-static void Dw_widget_update_cursor       (DwWidget *widget);
-
-
 enum
 {
-   SIZE_REQUEST,
-   FAST_SIZE_REQUEST,
-   SIZE_ALLOCATE,
-   SET_WIDTH,
-   SET_ASCENT,
-   SET_DESCENT,
-   DRAW,
-   REALIZE,
-   UNREALIZE,
    BUTTON_PRESS_EVENT,
    BUTTON_RELEASE_EVENT,
    MOTION_NOTIFY_EVENT,
@@ -122,68 +110,6 @@ static void Dw_widget_class_init (DwWidgetClass *klass)
 
    object_class = GTK_OBJECT_CLASS (klass);
 
-   widget_signals[SIZE_REQUEST] =
-      gtk_signal_new ("size_request",
-                      GTK_RUN_FIRST,
-                      object_class->type,
-                      GTK_SIGNAL_OFFSET (DwWidgetClass, size_request),
-                      gtk_marshal_NONE__POINTER,
-                      GTK_TYPE_NONE,
-                      1, GTK_TYPE_POINTER);
-   widget_signals[SIZE_ALLOCATE] =
-      gtk_signal_new ("size_allocate",
-                      GTK_RUN_FIRST,
-                      object_class->type,
-                      GTK_SIGNAL_OFFSET (DwWidgetClass, size_allocate),
-                      gtk_marshal_NONE__POINTER,
-                      GTK_TYPE_NONE,
-                      1, GTK_TYPE_POINTER);
-   widget_signals[SET_WIDTH] =
-      gtk_signal_new ("set_width",
-                      GTK_RUN_FIRST,
-                      object_class->type,
-                      GTK_SIGNAL_OFFSET (DwWidgetClass, set_width),
-                      gtk_marshal_NONE__INT,
-                      GTK_TYPE_NONE,
-                      1, GTK_TYPE_UINT);
-   widget_signals[SET_ASCENT] =
-      gtk_signal_new ("set_ascent",
-                      GTK_RUN_FIRST,
-                      object_class->type,
-                      GTK_SIGNAL_OFFSET (DwWidgetClass, set_ascent),
-                      gtk_marshal_NONE__INT,
-                      GTK_TYPE_NONE,
-                      1, GTK_TYPE_UINT);
-   widget_signals[SET_DESCENT] =
-      gtk_signal_new ("set_descent",
-                      GTK_RUN_FIRST,
-                      object_class->type,
-                      GTK_SIGNAL_OFFSET (DwWidgetClass, set_descent),
-                      gtk_marshal_NONE__INT,
-                      GTK_TYPE_NONE,
-                      1, GTK_TYPE_UINT);
-   widget_signals[DRAW] =
-      gtk_signal_new ("draw",
-                      GTK_RUN_FIRST,
-                      object_class->type,
-                      GTK_SIGNAL_OFFSET (DwWidgetClass, draw),
-                      gtk_marshal_NONE__POINTER_POINTER,
-                      GTK_TYPE_NONE,
-                      2, GTK_TYPE_POINTER, GTK_TYPE_GDK_EVENT);
-   widget_signals[REALIZE] =
-      gtk_signal_new ("realize",
-                      GTK_RUN_FIRST,
-                      object_class->type,
-                      GTK_SIGNAL_OFFSET (DwWidgetClass, realize),
-                      gtk_marshal_NONE__NONE,
-                      GTK_TYPE_NONE, 0);
-   widget_signals[UNREALIZE] =
-      gtk_signal_new ("unrealize",
-                      GTK_RUN_FIRST,
-                      object_class->type,
-                      GTK_SIGNAL_OFFSET (DwWidgetClass, unrealize),
-                      gtk_marshal_NONE__NONE,
-                      GTK_TYPE_NONE, 0);
   widget_signals[BUTTON_PRESS_EVENT] =
      gtk_signal_new ("button_press_event",
                      GTK_RUN_LAST,
@@ -238,8 +164,6 @@ static void Dw_widget_class_init (DwWidgetClass *klass)
    klass->set_ascent = NULL;
    klass->set_descent = NULL;
    klass->draw = NULL;
-   klass->realize = NULL;
-   klass->unrealize = NULL;
    klass->button_press_event = NULL;
    klass->button_release_event = NULL;
    klass->motion_notify_event = NULL;
@@ -259,8 +183,6 @@ static void Dw_widget_shutdown (GtkObject *object)
 
    widget = DW_WIDGET (object);
 
-   a_Dw_widget_unrealize (widget);
-
    if (widget->parent)
       Dw_container_remove (DW_CONTAINER (widget->parent), widget);
    else
@@ -315,7 +237,7 @@ static void Dw_widget_real_get_extremes (DwWidget *widget,
    /* Simply return the requisition width */
    DwRequisition requisition;
 
-   a_Dw_widget_size_request (widget, &requisition);
+   p_Dw_widget_size_request (widget, &requisition);
    extremes->min_width = extremes->max_width = requisition.width;
 }
 
@@ -324,13 +246,15 @@ static void Dw_widget_real_get_extremes (DwWidget *widget,
  * This function is a wrapper for DwWidget::size_request; it calls
  * this method only when needed.
  */
-void a_Dw_widget_size_request (DwWidget *widget,
+void p_Dw_widget_size_request (DwWidget *widget,
                                DwRequisition *requisition)
 {
+   DwWidgetClass *klass;
+
    if (DW_WIDGET_NEEDS_RESIZE (widget)) {
       /* todo: check requisition == &(widget->requisition) and do what? */
-      gtk_signal_emit (GTK_OBJECT (widget), widget_signals[SIZE_REQUEST],
-                       requisition);
+      klass =  DW_WIDGET_CLASS (GTK_OBJECT(widget)->klass);
+      (* (klass->size_request)) (widget, requisition);
       widget->requisition = *requisition;
       DW_WIDGET_UNSET_FLAGS (widget, DW_NEEDS_RESIZE);
 
@@ -346,7 +270,7 @@ void a_Dw_widget_size_request (DwWidget *widget,
 /*
  * Wrapper for DwWidget::get_extremes.
  */
-void a_Dw_widget_get_extremes (DwWidget *widget,
+void p_Dw_widget_get_extremes (DwWidget *widget,
                                DwExtremes *extremes)
 {
    DwWidgetClass *klass;
@@ -369,9 +293,11 @@ void a_Dw_widget_get_extremes (DwWidget *widget,
 /*
  * Wrapper for DwWidget::size_allocate, only called when needed.
  */
-void a_Dw_widget_size_allocate  (DwWidget *widget,
+void p_Dw_widget_size_allocate  (DwWidget *widget,
                                  DwAllocation *allocation)
 {
+   DwWidgetClass *klass;
+
    if (DW_WIDGET_NEEDS_ALLOCATE (widget) ||
        allocation->x != widget->allocation.x ||
        allocation->y != widget->allocation.y ||
@@ -388,8 +314,9 @@ void a_Dw_widget_size_allocate  (DwWidget *widget,
                  widget->allocation.width, widget->allocation.ascent,
                  widget->allocation.descent);
 
-      gtk_signal_emit (GTK_OBJECT (widget), widget_signals[SIZE_ALLOCATE],
-                       allocation);
+      klass =  DW_WIDGET_CLASS (GTK_OBJECT(widget)->klass);
+      if (klass->size_allocate)
+         (* (klass->size_allocate)) (widget, allocation);
 
       DEBUG_MSG (DEBUG_ALLOC, "... to %d, %d, %d x %d x %d\n",
                  widget->allocation.x, widget->allocation.y,
@@ -412,28 +339,37 @@ void a_Dw_widget_size_allocate  (DwWidget *widget,
 }
 
 
-void a_Dw_widget_set_width (DwWidget *widget,
+void p_Dw_widget_set_width (DwWidget *widget,
                             gint32 width)
 {
-   gtk_signal_emit (GTK_OBJECT (widget), widget_signals[SET_WIDTH], width);
+   DwWidgetClass *klass;
+   klass =  DW_WIDGET_CLASS (GTK_OBJECT(widget)->klass);
+   if (klass->set_width)
+      (* (klass->set_width)) (widget, width);
 }
 
 
-void a_Dw_widget_set_ascent (DwWidget *widget,
+void p_Dw_widget_set_ascent (DwWidget *widget,
                              gint32 ascent)
 {
-   gtk_signal_emit (GTK_OBJECT (widget), widget_signals[SET_ASCENT], ascent);
+   DwWidgetClass *klass;
+   klass =  DW_WIDGET_CLASS (GTK_OBJECT(widget)->klass);
+   if (klass->set_ascent)
+      (* (klass->set_ascent)) (widget, ascent);
 }
 
 
-void a_Dw_widget_set_descent (DwWidget *widget,
+void p_Dw_widget_set_descent (DwWidget *widget,
                               gint32 descent)
 {
-   gtk_signal_emit (GTK_OBJECT (widget), widget_signals[SET_DESCENT], descent);
+   DwWidgetClass *klass;
+   klass =  DW_WIDGET_CLASS (GTK_OBJECT(widget)->klass);
+   if (klass->set_descent)
+      (* (klass->set_descent)) (widget, descent);
 }
 
 
-void a_Dw_widget_draw (DwWidget *widget,
+void p_Dw_widget_draw (DwWidget *widget,
                        DwRectangle *area,
                        GdkEventExpose *event)
 {
@@ -443,11 +379,13 @@ void a_Dw_widget_draw (DwWidget *widget,
    GtkDwViewport *viewport;
    GdkDrawable *orig_pixmap, *dest;
    gint x, y;
+   DwWidgetClass *klass;
 
    orig_pixmap = widget->parent ? widget->parent->clip_pixmap : NULL;
    widget->clip_pixmap = orig_pixmap;
-   gtk_signal_emit (GTK_OBJECT (widget), widget_signals[DRAW],
-                    area, event);
+   klass =  DW_WIDGET_CLASS (GTK_OBJECT(widget)->klass);
+   if (klass->draw)
+      (* (klass->draw)) (widget, area, event);
 
    if (widget->clip_pixmap &&
        (widget->parent == NULL ||
@@ -468,41 +406,6 @@ void a_Dw_widget_draw (DwWidget *widget,
 }
 
 
-void a_Dw_widget_realize (DwWidget *widget)
-{
-   if (!DW_WIDGET_REALIZED (widget)) {
-      gtk_signal_emit (GTK_OBJECT (widget), widget_signals[REALIZE]);
-      DW_WIDGET_SET_FLAGS (widget, DW_REALIZED);
-
-      if (DW_IS_CONTAINER (widget))
-         a_Dw_container_forall (DW_CONTAINER (widget),
-                                (DwCallback) a_Dw_widget_realize,
-                                NULL);
-
-      Dw_widget_update_cursor (widget);
-
-      if (widget->parent == NULL && widget->style->background_color != NULL)
-         gdk_window_set_background (GTK_LAYOUT(widget->viewport)->bin_window,
-                                    &widget->style->background_color->color);
-   }
-}
-
-
-void a_Dw_widget_unrealize (DwWidget *widget)
-{
-   if (DW_WIDGET_REALIZED (widget)) {
-      a_Dw_widget_set_cursor (widget, NULL);
-      gtk_signal_emit (GTK_OBJECT (widget), widget_signals[UNREALIZE]);
-      DW_WIDGET_UNSET_FLAGS (widget, DW_REALIZED);
-
-      if (DW_IS_CONTAINER (widget))
-         a_Dw_container_forall (DW_CONTAINER (widget),
-                                (DwCallback) a_Dw_widget_unrealize,
-                                NULL);
-   }
-}
-
-
 /*
  * Handles a mouse event.
  *
@@ -686,18 +589,13 @@ void a_Dw_widget_set_style (DwWidget *widget,
    a_Dw_style_ref (style);
    widget->style = style;
 
-   if (widget->parent == NULL &&
-       DW_WIDGET_REALIZED (widget) &&
-       widget->style->background_color != NULL)
-      gdk_window_set_background (GTK_LAYOUT(widget->viewport)->bin_window,
-                                 &widget->style->background_color->color);
+   if (widget->parent == NULL && widget->viewport != NULL)
+      Dw_gtk_viewport_update_background (GTK_DW_VIEWPORT (widget->viewport));
 
-   if (DW_WIDGET_REALIZED (widget)) {
-      if (size_changed)
-         p_Dw_widget_queue_resize (widget, 0, TRUE);
-      else
-         p_Dw_widget_queue_draw (widget);
-   }
+   if (size_changed)
+      p_Dw_widget_queue_resize (widget, 0, TRUE);
+   else
+      p_Dw_widget_queue_draw (widget);
 }
 
 
@@ -705,7 +603,7 @@ void a_Dw_widget_set_style (DwWidget *widget,
  * Set the cursor of the viewport.
  * Called from several other functions.
  */
-static void Dw_widget_update_cursor (DwWidget *widget)
+void Dw_widget_update_cursor (DwWidget *widget)
 {
    GtkDwViewport *viewport = GTK_DW_VIEWPORT (widget->viewport);
    DwWidget *cursor_widget;
@@ -735,8 +633,7 @@ void a_Dw_widget_set_cursor (DwWidget *widget,
                              GdkCursor *cursor)
 {
    widget->cursor = cursor;
-   if (DW_WIDGET_REALIZED (widget))
-      Dw_widget_update_cursor (widget);
+   Dw_widget_update_cursor (widget);
 }
 
 
@@ -752,7 +649,7 @@ void a_Dw_widget_set_cursor (DwWidget *widget,
  * but this case is much more complicated, since a button is more complex
  * than a link.
  *
- * NOTE: This may be replaced by somehow using signals for events. 
+ * NOTE: This may be replaced by somehow using signals for events.
  */
 void a_Dw_widget_set_button_sensitive (DwWidget *widget,
                                        gboolean button_sensitive)
@@ -857,7 +754,7 @@ void p_Dw_iterator_get_allocation_std_only_widgets (DwIterator *it,
  * seen, according to hpos and vpos. The parameters start and end have
  * the same meaning as in DwIterator::get_allocation(); start refers
  * to it1, while end ferers to it2.
- * 
+ *
  * If it1 and it2 point to the same location (see code below), only
  * it1 is regarded, and both start and end refer to it.
  */
@@ -1201,7 +1098,7 @@ static void Dw_widget_print_tree0 (DwWidget *widget, int indent)
          MSG ("%*s%s\n", indent + 3, "", a_Dw_content_text (&it->content));
    }
 }
-   
+
 void a_Dw_widget_print_tree (DwWidget *widget)
 {
    MSG ("--- START OF WIDGET TREE ---\n");
@@ -1357,9 +1254,6 @@ void p_Dw_widget_set_parent (DwWidget *widget,
       widget->button_sensitive = parent->button_sensitive;
 
    DBG_OBJ_ASSOC (widget, parent);
-
-   if (DW_WIDGET_REALIZED (parent))
-      a_Dw_widget_realize (widget);
 }
 
 
@@ -1564,7 +1458,7 @@ void p_Dw_widget_will_clip (DwWidget *widget)
                       widget->viewport->allocation.width,
                       widget->viewport->allocation.height,
                       GTK_DW_VIEWPORT(widget->viewport)->depth);
-   
+
    /* Determine the effective background color. (There is a defined background
     * at least for the top-level widget.) */
    for (widget2 = widget;
