@@ -1,22 +1,20 @@
+/*
+ * File: gif.c
+ *
+ * Copyright (C) 1997 Raph Levien <raph@acm.org>
+ * Copyright (C) 2000-2002 Jorge Arellano Cid <jcid@dillo.org>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ */
+
 /*
  * The GIF decoder for dillo. It is responsible for decoding GIF data
  * and transferring it to the dicache.
- *
- * Adapted by Raph Levien from giftopnm.c as found in the
- * netpbm-1mar1994 release. The copyright notice for giftopnm.c
- * follows:
  */
 
-/* +-------------------------------------------------------------------+ */
-/* | Copyright 1990, 1991, 1993, David Koblas.  (koblas@netcom.com)    | */
-/* |   Permission to use, copy, modify, and distribute this software   | */
-/* |   and its documentation for any purpose and without fee is hereby | */
-/* |   granted, provided that the above copyright notice appear in all | */
-/* |   copies and that both that copyright notice and this permission  | */
-/* |   notice appear in supporting documentation.  This software is    | */
-/* |   provided "as is" without express or implied warranty.           | */
-/* +-------------------------------------------------------------------+ */
-
 
 /* Notes 13 Oct 1997 --RLL
  *
@@ -55,23 +53,29 @@
  * directly to 16 bits would help, but that's optimization again :)
  */
 
+
 /* todo:
  * + Make sure to handle error cases gracefully (including aborting the
  * connection, if necessary).
  */
 
-#undef VERBOSE
+#include <config.h>
+#ifdef ENABLE_GIF
 
 #include <stdio.h>              /* for sprintf */
 #include <string.h>             /* for memcpy and memmove */
 
 #include <gtk/gtk.h>
+#include "msg.h"
 #include "image.h"
 #include "web.h"
 #include "cache.h"
 #include "dicache.h"
 #include "prefs.h"
 
+#define DEBUG_LEVEL 6
+#include "debug.h"
+
 #define INTERLACE      0x40
 #define LOCALCOLORMAP  0x80
 
@@ -206,6 +210,7 @@ static DilloGif *Gif_new(DilloImage *Image, DilloUrl *url, gint version)
    gif->spill_lines = NULL;
    gif->window = 0;
    gif->packet_size = 0;
+   gif->ColorMap_ofs = 0;
 
    return gif;
 }
@@ -237,9 +242,7 @@ static void Gif_write(DilloGif *gif, void *Buf, gint BufSize)
    buf = ((guchar *) Buf) + gif->Start_Ofs;
    bufsize = BufSize - gif->Start_Ofs;
 
-#ifdef VERBOSE
-   g_print("Gif_write: %d bytes\n", BufSize);
-#endif
+   DEBUG_MSG(5, "Gif_write: %d bytes\n", BufSize);
 
    /* Process the bytes in the input buffer. */
    bytes_consumed = Gif_process_bytes(gif, buf, bufsize, Buf);
@@ -248,9 +251,7 @@ static void Gif_write(DilloGif *gif, void *Buf, gint BufSize)
       return;
    gif->Start_Ofs += bytes_consumed;
 
-#ifdef VERBOSE
-   g_print("exit Gif_write, bufsize=%d\n", bufsize);
-#endif
+   DEBUG_MSG(5, "exit Gif_write, bufsize=%d\n", bufsize);
 }
 
 /*
@@ -260,14 +261,11 @@ static void Gif_close(DilloGif *gif, CacheClient_t *Client)
 {
    gint i;
 
-#ifdef VERBOSE
-   g_print("destroy gif %x\n", gif);
-#endif
+   DEBUG_MSG(5, "destroy gif %p\n", gif);
 
    a_Dicache_close(gif->url, gif->version, Client);
 
-   if (gif->linebuf != NULL)
-      g_free(gif->linebuf);
+   g_free(gif->linebuf);
 
    if (gif->spill_lines != NULL) {
       for (i = 0; i < gif->num_spill_lines_max; i++)
@@ -391,10 +389,7 @@ static size_t Gif_do_extension(DilloGif *gif, guchar Label,
 }
 
 /* --- General Image Decoder ----------------------------------------------- */
-/* Here begins the new push-oriented decoder. It should be quite a bit
- * faster than the old code, which was adapted to be asynchronous from
- * David Koblas's original giftopnm code.
- */
+/* Here begins the new push-oriented decoder. */
 
 /*
  * ?
@@ -493,12 +488,11 @@ static void Gif_literal(DilloGif *gif, guint code)
 static void Gif_sequence(DilloGif *gif, guint code)
 {
    guint o_index, o_size, orig_code;
-   guchar *obuf = NULL;
    guint sequence_length = gif->length[code];
    guint line_index = gif->line_index;
    gint num_spill_lines;
    gint spill_line_index = gif->spill_line_index;
-   guchar *last_byte_ptr;
+   guchar *last_byte_ptr, *obuf;
 
    gif->length[gif->last_code + 1] = sequence_length + 1;
    gif->code_and_byte[gif->last_code + 1] = (code << 8);
@@ -548,9 +542,8 @@ static void Gif_sequence(DilloGif *gif, guint code)
       for (; o_size > 0 && o_index > 0; o_size--) {
          guint code_and_byte = gif->code_and_byte[code];
 
-#ifdef VERBOSE
-         printf("%d ", gif->code_and_byte[code] & 255);
-#endif
+         DEBUG_MSG(5, "%d ", gif->code_and_byte[code] & 255);
+
          obuf[--o_index] = code_and_byte & 255;
          code = code_and_byte >> 8;
       }
@@ -569,25 +562,18 @@ static void Gif_sequence(DilloGif *gif, guint code)
    }
    /* Ok, now we write the first byte of the sequence. */
    /* We are sure that the code is literal. */
-#ifdef VERBOSE
-   printf("%d", code);
-#endif
+   DEBUG_MSG(5, "%d", code);
    obuf[--o_index] = code;
    gif->code_and_byte[gif->last_code] |= code;
 
    /* Fix up the output if the original code was last_code. */
    if (orig_code == gif->last_code) {
       *last_byte_ptr = code;
-#ifdef VERBOSE
-      printf(" fixed (%d)!", code);
-#endif
+      DEBUG_MSG(5, " fixed (%d)!", code);
    }
-#ifdef VERBOSE
-   printf("\n");
-#endif
+   DEBUG_MSG(5, "\n");
 
    /* Output any full lines. */
-   o_index = gif->line_index;
    if (gif->line_index >= gif->Width) {
       Gif_emit_line(gif, gif->linebuf);
       gif->line_index = 0;
@@ -596,7 +582,7 @@ static void Gif_sequence(DilloGif *gif, guint code)
       if (gif->line_index)
          Gif_emit_line(gif, gif->linebuf);
       for (spill_line_index = 0;
-           spill_line_index < num_spill_lines - (o_index ? 1 : 0);
+           spill_line_index < num_spill_lines - (gif->line_index ? 1 : 0);
            spill_line_index++)
          Gif_emit_line(gif, gif->spill_lines[spill_line_index]);
    }
@@ -626,15 +612,13 @@ static gint Gif_process_code(DilloGif *gif, guint code, guint clear_code)
 
    /* A short table describing what to do with the code:
     * code < clear_code  : This is uncompressed, raw data
-    * code== clear_code  : Reset the decmpression table
+    * code== clear_code  : Reset the decompression table
     * code== clear_code+1: End of data stream
     * code > clear_code+1: Compressed code; look up in table
     */
    if (code < clear_code) {
       /* a literal code. */
-#ifdef VERBOSE
-      printf("literal\n");
-#endif
+      DEBUG_MSG(5, "literal\n");
       Gif_literal(gif, code);
       return 1;
    } else if (code >= clear_code + 2) {
@@ -645,15 +629,11 @@ static gint Gif_process_code(DilloGif *gif, guint code, guint clear_code)
       return 1;
    } else if (code == clear_code) {
       /* clear code. Resets the whole table */
-#ifdef VERBOSE
-      printf("clear\n");
-#endif
+      DEBUG_MSG(5, "clear\n");
       return 0;
    } else {
       /* end code. */
-#ifdef VERBOSE
-      printf("end\n");
-#endif
+      DEBUG_MSG(5, "end\n");
       return 2;
    }
 }
@@ -661,7 +641,7 @@ static gint Gif_process_code(DilloGif *gif, guint code, guint clear_code)
 /*
  * ?
  */
-static gint Gif_decode(DilloGif *gif, const guchar *buf, size_t BSize)
+static gint Gif_decode(DilloGif *gif, const guchar *buf, size_t bsize)
 {
    /*
     * Data block processing.  The image stuff is a series of data blocks.
@@ -676,7 +656,7 @@ static gint Gif_decode(DilloGif *gif, const guchar *buf, size_t BSize)
    gint code_size;
    guint code_mask;
 
-   bufsize = BSize;
+   bufsize = bsize;
 
    /* Want to get all inner loop state into local variables. */
    packet_size = gif->packet_size;
@@ -711,10 +691,8 @@ static gint Gif_decode(DilloGif *gif, const guchar *buf, size_t BSize)
              * at the start of the window */
             code = (window >> (32 - bits_in_window)) & code_mask;
 
-            /* printf ("%d\n", code); would print the code stream. */
-#ifdef VERBOSE
-            printf("last_code = %d, code = %d, ", last_code, code);
-#endif
+            DEBUG_MSG(5, "code = %d, ", code);
+
             bits_in_window -= code_size;
             switch (Gif_process_code(gif, code, clear_code)) {
             case 1:             /* Increment last code */
@@ -766,20 +744,20 @@ static gint Gif_decode(DilloGif *gif, const guchar *buf, size_t BSize)
    gif->window = window;
    gif->bits_in_window = bits_in_window;
    gif->code_size = code_size;
-   return BSize - bufsize;
+   return bsize - bufsize;
 
  error:
    gif->state = 999;
-   return BSize - bufsize;
+   return bsize - bufsize;
 }
 
 /*
  * ?
  */
-static gint Gif_check_sig(DilloGif *gif, const guchar *ibuf, size_t Size)
+static gint Gif_check_sig(DilloGif *gif, const guchar *ibuf, size_t ibsize)
 {
    /* at beginning of file - read magic number */
-   if (Size < 6)
+   if (ibsize < 6)
       return 0;
    if (strncmp(ibuf, "GIF", 3) != 0) {
       gif->state = 999;
@@ -802,11 +780,11 @@ static gint Gif_check_sig(DilloGif *gif, const guchar *ibuf, size_t Size)
  */
 static inline size_t
  Gif_do_color_table(DilloGif *gif, void *Buf,
-                    const guchar *buf, size_t BSize, size_t CT_Size)
+                    const guchar *buf, size_t bsize, size_t CT_Size)
 {
    size_t Size = 3 * (1 << (1 + CT_Size));
 
-   if (Size > BSize)
+   if (Size > bsize)
       return 0;
 
    gif->ColorMap_ofs = (gulong) buf - (gulong) Buf;
@@ -819,7 +797,7 @@ static inline size_t
  * <Logical Screen> ::= Logical Screen Descriptor [Global Color Table]
  */
 static size_t Gif_get_descriptor(DilloGif *gif, void *Buf,
-                                 const guchar *buf, size_t BSize)
+                                 const guchar *buf, size_t bsize)
 {
 
    /* screen descriptor */
@@ -827,12 +805,12 @@ static size_t Gif_get_descriptor(DilloGif *gif, void *Buf,
           mysize;             /* Size of color table */
    guchar Flags;
 
-   if (BSize < 7)
+   if (bsize < 7)
       return 0;
    Flags = buf[4];
 
    if (Flags & LOCALCOLORMAP) {
-      mysize = Gif_do_color_table(gif, Buf, buf + 7, BSize - 7, Flags & 0x7);
+      mysize = Gif_do_color_table(gif, Buf, buf + 7, bsize - 7, Flags & 0x7);
       if (!mysize)
          return 0;
       Size += mysize;           /* Size of the color table that follows */
@@ -900,7 +878,7 @@ static size_t Gif_do_img_desc(DilloGif *gif, void *Buf,
    if (gif->Image && gif->ColorMap_ofs) {
       a_Dicache_set_cmap(gif->url, gif->version, gif->Image,
                          (guchar *) Buf + gif->ColorMap_ofs,
-                         256 /* gif->NumColors */, gif->transparent);
+                         gif->NumColors, 256, gif->transparent);
    }
    return Size;
 }
@@ -985,8 +963,8 @@ static size_t GIF_Block(DilloGif * gif, void *Buf,
          break;                 /* GIF terminator */
 
       default:                  /* Unknown */
-         /* BUG: we should gripe and complain
-          * g_print(0x%x\n", *(buf-1)); */
+         /* gripe and complain */
+         MSG ("gif.c::GIF_Block: Error, 0x%x found\n", *(buf-1));
          gif->state = 999;
          return Size + 1;
       }
@@ -1061,10 +1039,11 @@ static size_t Gif_process_bytes(DilloGif *gif, const guchar *ibuf,
       tmp_bufsize = 0;
       break;
    }
-#ifdef VERBOSE
-   g_print("Gif_process_bytes: initial state %d, final state %d,"
-           "%d bytes consumed\n",
-           initial_state, gif->state, bufsize - tmp_bufsize);
-#endif
+
+   DEBUG_MSG(5, "Gif_process_bytes: final state %d, %d bytes consumed\n",
+             gif->state, bufsize - tmp_bufsize);
+
    return bufsize - tmp_bufsize;
 }
+
+#endif /* ENABLE_GIF */
