@@ -31,17 +31,17 @@ typedef struct _DilloHtmlInput   DilloHtmlInput;
 struct _DilloHtmlLB {
    BrowserWindow *bw;
    DilloUrl *base_url;
- 
+
    DilloHtmlForm *forms;
    gint num_forms;
    gint num_forms_max;
- 
+
    DilloUrl **links;
    gint num_links;
    gint num_links_max;
- 
+
    DwImageMapList maps;
- 
+
    gint32 link_color;
    gint32 visited_color;
 
@@ -104,10 +104,10 @@ struct _DilloHtmlState {
    gboolean cell_text_align_set;
    enum { HTML_LIST_NONE, HTML_LIST_UNORDERED, HTML_LIST_ORDERED } list_type;
    gint list_number;
- 
+
    /* TagInfo index for the tag that's being processed */
    gint tag_idx;
- 
+
    DwWidget *page, *table;
 
    /* This is used to align list items (especially in enumerated lists) */
@@ -137,7 +137,7 @@ struct _DilloHtmlForm {
    DilloHtmlMethod method;
    DilloUrl *action;
    DilloHtmlEnc enc;
- 
+
    DilloHtmlInput *inputs;
    gint num_inputs;
    gint num_inputs_max;
@@ -154,7 +154,7 @@ struct _DilloHtmlOption {
 struct _DilloHtmlSelect {
    GtkWidget *menu;
    gint size;
- 
+
    DilloHtmlOption *options;
    gint num_options;
    gint num_options_max;
@@ -192,29 +192,33 @@ struct _DilloHtmlInput {
 
 struct _DilloHtml {
    DwWidget *dw;          /* this is duplicated in the stack (page) */
- 
+
    DilloHtmlLB *linkblock;
    gchar *Start_Buf;
    size_t Start_Ofs;
    size_t CurrTagOfs, CurrTagIdx;
    size_t OldTagOfs, OldTagLine;
- 
+
    DilloHtmlState *stack;
    gint stack_top;        /* Index to the top of the stack [0 based] */
    gint stack_max;
- 
+
    DilloHtmlProcessingState InFlags; /* tracks which tags we are in */
- 
+
    GString *Stash;
    gboolean StashSpace;
- 
-   gint pre_column;        /* current column, used in PRE tags with tabs */
-   gboolean PreFirstChar;  /* used to skip the first CR or CRLF in PRE tags*/
-   gboolean PrevWasCR;     /* Flag to help parsing of "\r\n" in PRE tags */
-   gboolean InVisitedLink; /* used to 'force_visited_colors' */
- 
+
+   gchar *SPCBuf;           /* Buffer for white space */
+
+   gint pre_column;         /* current column, used in PRE tags with tabs */
+   gboolean PreFirstChar;   /* used to skip the first CR or CRLF in PRE tags */
+   gboolean PrevWasCR;      /* Flag to help parsing of "\r\n" in PRE tags */
+   gboolean PrevWasOpenTag; /* Flag to help deferred parsing of white space */
+   gboolean SPCPending;     /* Flag to help deferred parsing of white space */
+   gboolean InVisitedLink;  /* used to 'contrast_visited_colors' */
+
    GString *attr_data;
- 
+
    BrowserWindow *bw;
 };
 
