@@ -19,6 +19,7 @@
 #include <stdio.h>
 #include <string.h>
 
+#include "msg.h"
 #include "image.h"
 
 /*
@@ -31,13 +32,15 @@ static guchar *linebuf = NULL;
 /*
  * Create and initialize a new image structure.
  */
-DilloImage *
- a_Image_new(gint width, gint height, const char *alt, gint32 bg_color)
+DilloImage *a_Image_new(gint width,
+                        gint height,
+                        const char *alt_text,
+                        gint32 bg_color)
 {
    DilloImage *Image;
 
    Image = g_new(DilloImage, 1);
-   Image->dw = (DwImage *) a_Dw_image_new(DW_IMAGE_RGB, alt);
+   Image->dw = (DwImage *) a_Dw_image_new(DW_IMAGE_RGB, alt_text);
    Image->width = 0;
    Image->height = 0;
    Image->cmap = NULL;
@@ -66,7 +69,7 @@ static void Image_free(DilloImage *Image)
  */
 void a_Image_unref(DilloImage *Image)
 {
-   /* g_print(" %d ", Image->RefCount); */
+   _MSG(" %d ", Image->RefCount);
    if (Image && --Image->RefCount == 0)
       Image_free(Image);
 }
