@@ -35,6 +35,7 @@
 #include "prefs.h"
 #include "url.h"
 #include "capi.h"
+#include "gtk_ext_button.h"
 
 #include "dw_widget.h"
 #include "dw_gtk_scrolled_window.h"
@@ -61,6 +62,12 @@ static gint num_bw, num_bw_max;
 /* We need only one of them. */
 static GtkTooltips *tooltips = NULL;
 
+/* open dialog last dir */
+static gchar *open_dialog_last_dirname = NULL;
+
+/* save dialog last dir */
+static gchar *save_dialog_last_dirname = NULL;
+
 
 /*
  * Initialize global data
@@ -71,6 +78,8 @@ void a_Interface_init(void)
    num_bw_max = 16;
    browser_window = NULL;
    tooltips = gtk_tooltips_new ();
+   open_dialog_last_dirname = NULL;
+   save_dialog_last_dirname = NULL;
 }
 
 /*
@@ -333,15 +342,56 @@ static gboolean Interface_quit(GtkWidget *widget, BrowserWindow *bw)
 
 
 /*=== Browser Window Interface Construction =================================*/
+
 /*
  * Clear a text entry
  */
 static void Interface_entry_clear(GtkEntry *entry)
 {
-   gtk_entry_set_text(GTK_ENTRY (entry), "");
+   gtk_entry_set_text(entry, "");
    gtk_widget_grab_focus(GTK_WIDGET(entry));
 }
 
+/*
+ * Get the selection into the clear url button.
+ * (cub = clear url button)
+ */
+static void Interface_cub_get_selection(GtkWidget *widget, gpointer data)
+{
+  /* Request the the primary selection as a string */
+  gtk_selection_convert (widget, GDK_SELECTION_PRIMARY,
+                         GDK_SELECTION_TYPE_STRING,
+                         GDK_CURRENT_TIME);
+}
+
+/*
+ * Receive the selection (from a paste event)
+ * (cub = clear url button)
+ */
+static void Interface_cub_selection_received(GtkWidget *widget,
+                                             GtkSelectionData *selection_data,
+                                             guint32 time,
+                                             gpointer data)
+{
+   BrowserWindow *bw = data;
+   gchar *damn_string;
+
+   _MSG("Interface_cub_selection_received:\n");
+
+   if (selection_data->length < 0) {
+       DEBUG_MSG (1, "Selection retrieval failed\n");
+       return;
+   }
+   
+   damn_string = g_strndup((gchar *)selection_data->data,
+                           selection_data->length);
+   gtk_entry_set_text(GTK_ENTRY(bw->location), damn_string);
+   gtk_widget_activate(GTK_WIDGET(bw->location));
+   g_free(damn_string);
+
+   return;
+}
+
 /*
  * Create a pixmap and return it.
  */
@@ -362,6 +412,45 @@ static GtkWidget *Interface_pixmap_new(GtkWidget *parent, gchar **data)
    return (pixmapwid);
 }
 
+/*
+ * Create an extended button for the toolbar. "label_text" may be NULL.
+ * "icon_ret" and "label_ret " may be used to return the child widgets, but
+ * may also be 0.
+ */
+static GtkWidget *Interface_toolbox_ext_button_new(GtkWidget *parent,
+                                                   gchar *label_text,
+                                                   gchar **image_data,
+                                                   GtkWidget **icon_ret,
+                                                   GtkWidget **label_ret)
+{
+   GtkWidget *button, *pixmap, *vbox, *label;
+
+   button = a_Gtk_ext_button_new ();
+   GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
+   gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
+   pixmap = Interface_pixmap_new (parent, image_data);
+
+   if (label_text) {
+      vbox = gtk_vbox_new(FALSE, 0);
+      gtk_container_add(GTK_CONTAINER (button), vbox);
+      gtk_box_pack_start(GTK_BOX(vbox), pixmap, FALSE, FALSE, 0);
+      label = gtk_label_new(label_text);
+      gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
+   } else {
+      gtk_container_add(GTK_CONTAINER (button), pixmap);
+      label = NULL;
+   }
+
+   gtk_widget_show_all (button);
+
+   if (icon_ret)
+      *icon_ret = pixmap;
+   if (label_ret)
+      *label_ret = label;
+
+   return button;
+}
+
 /*
  * Set the bw's cursor type
  */
@@ -411,14 +500,27 @@ static GtkWidget *Interface_locbar_new(BrowserWindow *bw)
    gtk_toolbar_set_button_relief(GTK_TOOLBAR(toolbar_l), GTK_RELIEF_NONE);
    GTK_WIDGET_UNSET_FLAGS (toolbar_l, GTK_CAN_FOCUS);
 
-   bw->clear_url_button = gtk_toolbar_append_item(
-                             GTK_TOOLBAR(toolbar_l),
-                             NULL, "Clear the url-box!", "Toolbar/New",
-                             Interface_pixmap_new(bw->main_window, s_new_xpm),
-                             NULL, NULL);
+   bw->clear_url_button =
+      Interface_toolbox_ext_button_new(bw->main_window, NULL, s_new_xpm,
+                                       NULL, NULL);
+   a_Gtk_ext_button_set_command(GTK_EXT_BUTTON(bw->clear_url_button), 1);
+   a_Gtk_ext_button_set_command(GTK_EXT_BUTTON(bw->clear_url_button), 2);
+
+   gtk_toolbar_append_widget(GTK_TOOLBAR(toolbar_l), bw->clear_url_button,
+                             "Clear the URL box (middle-click to paste an "
+                             "URL).", "Toolbar/Clear");
    gtk_signal_connect_object(GTK_OBJECT(bw->clear_url_button), "clicked",
                              GTK_SIGNAL_FUNC (Interface_entry_clear),
                              GTK_OBJECT(bw->location));
+   gtk_signal_connect_object(GTK_OBJECT(bw->clear_url_button), "clicked1",
+                             GTK_SIGNAL_FUNC (Interface_entry_clear),
+                             GTK_OBJECT(bw->location));
+   gtk_signal_connect(GTK_OBJECT(bw->clear_url_button), "clicked2",
+                      GTK_SIGNAL_FUNC (Interface_cub_get_selection),
+                      NULL);
+   gtk_signal_connect(GTK_OBJECT(bw->clear_url_button), "selection_received",
+                      GTK_SIGNAL_FUNC (Interface_cub_selection_received),
+                      bw);
 
    /* right toolbar (Search) */
    toolbar_r = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
@@ -427,7 +529,7 @@ static GtkWidget *Interface_locbar_new(BrowserWindow *bw)
 
    bw->search_button = gtk_toolbar_append_item(
                           GTK_TOOLBAR(toolbar_r),
-                          NULL, "Search the Web", "toolbar/New",
+                          NULL, "Search the Web", "Toolbar/New",
                           Interface_pixmap_new(bw->main_window, search_xpm),
                           GTK_SIGNAL_FUNC(a_Interface_search_dialog),
                           bw);
@@ -450,39 +552,75 @@ static GtkWidget *Interface_locbar_new(BrowserWindow *bw)
  */
 static GtkWidget *Interface_toolbar_new(BrowserWindow *bw, gint label)
 {
-   GtkWidget *toolbar;
+   GtkWidget *toolbar, *label_widget, *icon_widget;
+   GtkToolbarChild *toolbar_child;
    gboolean s = prefs.small_icons;
 
    toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
    gtk_toolbar_set_button_relief(GTK_TOOLBAR(toolbar), GTK_RELIEF_NONE);
 
    /* back button */
-   bw->back_button = gtk_toolbar_append_item(
-                        GTK_TOOLBAR(toolbar),
-                        label ? "Back" : NULL,
-                        "Go to previous page", "Toolbar/Back",
-                        Interface_pixmap_new(bw->main_window,
-                                             s ? s_left_xpm : left_xpm),
-                        (GtkSignalFunc) a_Commands_back_callback, bw);
+   bw->back_button =
+      Interface_toolbox_ext_button_new(bw->main_window, label ? "Back" : NULL,
+                                       s ? s_left_xpm : left_xpm,
+                                       &label_widget, &icon_widget);
+   gtk_toolbar_append_widget(GTK_TOOLBAR(toolbar), bw->back_button,
+                             "Go to previous page (right-click for menu).",
+                             "Toolbar/Back");
    gtk_widget_set_sensitive(bw->back_button, FALSE);
    Interface_set_button_accel(GTK_BUTTON(bw->back_button), GDK_comma,
                               0, bw->accel_group);
-   gtk_signal_connect(GTK_OBJECT(bw->back_button), "button-press-event",
-                      GTK_SIGNAL_FUNC(a_Commands_navpress_callback), bw);
+   a_Gtk_ext_button_set_command(GTK_EXT_BUTTON(bw->back_button), 1);
+   a_Gtk_ext_button_attach_menu_creator(
+      GTK_EXT_BUTTON(bw->back_button), 3,
+      a_Commands_back_button_menu_creator_callback, (gpointer)bw);
+   gtk_signal_connect(GTK_OBJECT(bw->back_button), "clicked",
+                      GTK_SIGNAL_FUNC(a_Commands_back_callback), (gpointer)bw);
+   gtk_signal_connect(GTK_OBJECT(bw->back_button), "clicked1",
+                      GTK_SIGNAL_FUNC(a_Commands_back_callback), (gpointer)bw);
+
+   /*
+    * HACK: We have added GtkExtButton's as widgets
+    * (gtk_toolbar_append_widget), but they should behave like buttons etc.
+    * (gtk_toolbar_append_item), especially, they should have the same size.
+    * So we change some parts of the internal representation.
+    */
+   toolbar_child = ((GtkToolbarChild*)GTK_TOOLBAR(toolbar)->children->data);
+   toolbar_child->type = GTK_TOOLBAR_CHILD_BUTTON;
+   toolbar_child->widget = bw->back_button;
+   toolbar_child->icon = icon_widget;
+   toolbar_child->label = label_widget;
 
    /* forward button */
-   bw->forw_button = gtk_toolbar_append_item(
-                        GTK_TOOLBAR(toolbar),
-                        label ? "Forward" : NULL,
-                        "Go to next page", "Toolbar/Forward",
-                        Interface_pixmap_new(bw->main_window,
-                                             s ? s_right_xpm : right_xpm),
-                        (GtkSignalFunc) a_Commands_forw_callback, bw);
+   bw->forw_button =
+      Interface_toolbox_ext_button_new(bw->main_window,
+                                       label ? "Forward" : NULL,
+                                       s ? s_right_xpm : right_xpm,
+                                       &label_widget, &icon_widget);
+   gtk_toolbar_append_widget(GTK_TOOLBAR(toolbar), bw->forw_button,
+                             "Go to next page (right-click for menu).",
+                             "Toolbar/Forward");
    gtk_widget_set_sensitive(bw->forw_button, FALSE);
    Interface_set_button_accel(GTK_BUTTON(bw->forw_button), GDK_period,
                               0, bw->accel_group);
-   gtk_signal_connect(GTK_OBJECT(bw->forw_button), "button-press-event",
-                      GTK_SIGNAL_FUNC(a_Commands_navpress_callback), bw);
+   a_Gtk_ext_button_set_command(GTK_EXT_BUTTON(bw->forw_button), 1);
+   a_Gtk_ext_button_attach_menu_creator(
+      GTK_EXT_BUTTON(bw->forw_button), 3,
+      a_Commands_forw_button_menu_creator_callback, (gpointer)bw);
+   gtk_signal_connect(GTK_OBJECT(bw->forw_button), "clicked",
+                      GTK_SIGNAL_FUNC(a_Commands_forw_callback), (gpointer)bw);
+   gtk_signal_connect(GTK_OBJECT(bw->forw_button), "clicked1",
+                      GTK_SIGNAL_FUNC(a_Commands_forw_callback), (gpointer)bw);
+
+   /*
+    * HACK: See above.
+    */
+   toolbar_child =
+      ((GtkToolbarChild*)GTK_TOOLBAR(toolbar)->children->next->data);
+   toolbar_child->type = GTK_TOOLBAR_CHILD_BUTTON;
+   toolbar_child->widget = bw->forw_button;
+   toolbar_child->icon = icon_widget;
+   toolbar_child->label = label_widget;
 
    /* home button */
    bw->home_button = gtk_toolbar_append_item(
@@ -492,8 +630,11 @@ static GtkWidget *Interface_toolbar_new(BrowserWindow *bw, gint label)
                         Interface_pixmap_new(bw->main_window,
                                              s ? s_home_xpm : home_xpm),
                         (GtkSignalFunc) a_Commands_home_callback, bw);
-   gtk_signal_connect(GTK_OBJECT(bw->home_button), "button-press-event",
-                      GTK_SIGNAL_FUNC(a_Commands_navpress_callback), bw);
+   /*
+    * SG: This had never any effect, was it there for future extensions?
+    * gtk_signal_connect(GTK_OBJECT(bw->home_button), "button-press-event",
+    *                    GTK_SIGNAL_FUNC(a_Commands_navpress_callback), bw);
+    */
 
    /* reload button */
    bw->reload_button = gtk_toolbar_append_item(
@@ -537,8 +678,12 @@ static GtkWidget *Interface_toolbar_new(BrowserWindow *bw, gint label)
                               0, bw->accel_group);
    Interface_set_button_accel(GTK_BUTTON(bw->bookmarks_button), GDK_b,
                               GDK_CONTROL_MASK, bw->accel_group);
-   gtk_signal_connect(GTK_OBJECT(bw->bookmarks_button), "button-press-event",
-                      GTK_SIGNAL_FUNC(a_Commands_navpress_callback), bw);
+   /*
+    * SG: This had never any effect, was it there for future extensions?
+    * gtk_signal_connect(GTK_OBJECT(bw->bookmarks_button),
+    *                    "button-press-event",
+    *                    GTK_SIGNAL_FUNC(a_Commands_navpress_callback), bw);
+    */
 
    gtk_widget_show(toolbar);
    return toolbar;
@@ -679,7 +824,6 @@ static void Interface_add_full_screen_button (BrowserWindow *bw, GtkBox *box)
    gtk_widget_show (dummy);
 
    button = gtk_button_new ();
-   gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
    gtk_tooltips_set_tip (tooltips, button, "Hide Controls", "Show Controls");
    GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
    gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
@@ -863,9 +1007,9 @@ a_Interface_browser_window_new(gint width, gint height, guint32 xid)
                                    GTK_SIGNAL_FUNC(Interface_click_callback),
                                    (gpointer)bw);
 
+   /* full screen button: actually there're two buttons, one in the scrolled
+    * window (fs off) and one in the status bar (fs on). They look as one. */
    bw->full_screen_off_button = gtk_button_new ();
-   gtk_button_set_relief (GTK_BUTTON (bw->full_screen_off_button),
-                          GTK_RELIEF_NONE);
    gtk_tooltips_set_tip (tooltips, bw->full_screen_off_button,
                          "Show Controls", "Hide Controls");
    GTK_WIDGET_UNSET_FLAGS (bw->full_screen_off_button, GTK_CAN_FOCUS);
@@ -874,7 +1018,6 @@ a_Interface_browser_window_new(gint width, gint height, guint32 xid)
    pixmap = Interface_pixmap_new(bw->main_window, full_screen_off_xpm);
    gtk_container_add (GTK_CONTAINER (bw->full_screen_off_button), pixmap);
    gtk_widget_show (pixmap);
-
    gtk_signal_connect_object
       (GTK_OBJECT (bw->full_screen_off_button), "clicked",
        GTK_SIGNAL_FUNC (Interface_full_screen_callback), (gpointer)bw);
@@ -889,12 +1032,19 @@ a_Interface_browser_window_new(gint width, gint height, guint32 xid)
    gtk_widget_set_usize(bw->main_window, width, height);
 
    /* status widget */
+   /* create the over-bug-meter menu */
+   bw->menu_popup.over_bug = a_Menu_popup_ob_new(bw);
+
    bw->status = a_Dw_gtk_statuslabel_new("");
    gtk_misc_set_alignment(GTK_MISC(bw->status), 0.0, 0.5);
    /*  status widget for HTML errors.
     *  Note: the "clicked" signal is hooked with the linkblock later */
-   button = bw->status_bug_meter = gtk_button_new();
+   button = bw->status_bug_meter = a_Gtk_ext_button_new();
    GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
+   a_Gtk_ext_button_set_command (GTK_EXT_BUTTON (button), 1);
+   a_Gtk_ext_button_attach_menu (GTK_EXT_BUTTON (button), 3,
+                                 GTK_MENU(bw->menu_popup.over_bug));
+
    hbox = gtk_hbox_new(FALSE, 0);
    pixmap = Interface_pixmap_new(bw->main_window, mini_bug_xpm);
    gtk_object_set_data(GTK_OBJECT (button), "bug", pixmap);
@@ -908,7 +1058,8 @@ a_Interface_browser_window_new(gint width, gint height, guint32 xid)
    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
    gtk_container_add (GTK_CONTAINER (button), hbox);
    gtk_widget_show(hbox);
-   gtk_tooltips_set_tip (tooltips, button, "Show HTML bugs", "");
+   gtk_tooltips_set_tip (tooltips, button,
+                         "Show HTML bugs (right-click for menu).", "");
 
    bw->status_box = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(bw->status_box), bw->status, TRUE, TRUE, 2);
@@ -934,11 +1085,12 @@ a_Interface_browser_window_new(gint width, gint height, guint32 xid)
    bw->menu_popup.over_image = a_Menu_popup_oi_new(bw);
    bw->menu_popup.over_link = a_Menu_popup_ol_new(bw);
    bw->menu_popup.over_page = a_Menu_popup_op_new(bw);
-   bw->menu_popup.over_bug  = a_Menu_popup_ob_new(bw);
    bw->menu_popup.over_back = NULL;
    bw->menu_popup.over_forw = NULL;
    bw->menu_popup.url = NULL;
    bw->menu_popup.url2 = NULL;
+
+   bw->redirect_level = 0;
    bw->sens_idle_id = 0;
 
    bw->CursorType = -1;
@@ -1073,8 +1225,12 @@ void a_Interface_bug_meter_update(BrowserWindow *bw, gint num_err)
  */
 static void Interface_destroy_window(GtkWidget *widget, GtkWidget **window)
 {
-   gtk_widget_destroy(*window);
-   *window = NULL;
+   /* todo: sometimes this function is called twice with dialog windows */
+   _MSG("Interface_destroy_window %p\n", *window);
+   if (*window) {
+      gtk_widget_destroy(*window);
+      *window = NULL;
+   }
 }
 
 
@@ -1098,6 +1254,9 @@ void a_Interface_quit_all(void)
       gtk_widget_destroy(bws[i]->main_window);
 
    g_free(bws);
+   
+   g_free(open_dialog_last_dirname);
+   g_free(save_dialog_last_dirname);
 }
 
 /*
@@ -1157,45 +1316,43 @@ static void
    g_string_free(UrlStr, TRUE);
    g_free(s);
 
+   g_free(open_dialog_last_dirname);
+   open_dialog_last_dirname = g_strdup(fn);
+
    gtk_widget_destroy(bw->openfile_dialog_window);
 }
 
-/* 
- * Returns a newly allocated string holding a search url generated from 
+/*
+ * Returns a newly allocated string holding a search url generated from
  * a string of keywords (separarated by blanks) and prefs.search_url.
- * Any '"' or '+' in the keywords are escaped. 
+ * The search string is urlencoded.
  */
 static gchar *Interface_make_search_url(const gchar *str)
 {
-   gchar *p, *keys, *new_str;
+   gchar *keys = a_Url_encode_hex_str(str), *c = prefs.search_url;
+   GString *newstr = g_string_sized_new(strlen(c)+strlen(keys));
+   gchar *search_url;
 
-   keys = g_new(gchar, 3*strlen(str)+1);
+   for (; *c; c++)
+      if (*c == '%')
+         switch(*++c) {
+         case 's':
+            g_string_append(newstr, keys); break;;
+         case '%':
+            g_string_append_c(newstr, '%'); break;;
+         case 0:
+            MSG("Warning: search_url ends with '%%'\n"); c--; break;;
+         default:
+            MSG("Warning: illegal specifier '%%%c' in search_url\n", *c);
+         }
+      else
+         g_string_append_c(newstr, *c);
 
-   for (p = keys; *str; str++) {
-      switch(*str) {
-      case ' ':
-         *p++ = '+';
-         break;
-      case '"':
-         *p++ = '%';
-         *p++ = '2';
-         *p++ = '2';
-         break;
-      case '+':
-         *p++ = '%';
-         *p++ = '2';
-         *p++ = 'B';
-         break;
-      default:
-         *p++ = *str;
-      }
-   }
-   *p = '\0';
-
-   new_str = g_strdup_printf(prefs.search_url, keys);
    g_free(keys);
 
-   return new_str;
+   search_url = newstr->str;
+   g_string_free(newstr, FALSE);
+   return search_url;
 }
 
 /*
@@ -1255,6 +1412,11 @@ void a_Interface_openfile_dialog(BrowserWindow *bw)
          (GtkSignalFunc) Interface_openfile_ok_callback, (void *)bw);
    }
 
+   if (open_dialog_last_dirname)
+      gtk_file_selection_set_filename(
+         GTK_FILE_SELECTION(bw->openfile_dialog_window),
+         open_dialog_last_dirname);
+
    if (!GTK_WIDGET_VISIBLE(bw->openfile_dialog_window))
       gtk_widget_show(bw->openfile_dialog_window);
    else
@@ -1344,23 +1506,42 @@ static void
 }
 
 /*
- * Make a question-dialog with a question, OK and Cancel.
+ * Set bw->question_dialog_answer
+ */
+static void Interface_question_dialog_set_answer_cb(DialogAnswer *answer)
+{
+   _MSG("Interface_question_dialog_set_answer_cb\n");
+   answer->bw->question_dialog_answer = answer;
+}
+
+/*
+ * Make a question-dialog with a question and some alternatives.
+ * The selected choices are left in bw->question_dialog_answer in a structure.
+ * A generic callback function can decide afterwards based on that information.
+ * (0 means the window was cancelled, and 1 to 5 the respective alternatives)
  */
 static void Interface_make_question_dialog(
+        BrowserWindow *bw,
         GtkWidget **DialogWindow, char *WmName, char *WmClass,
-        char *WTitle, char *Question,
-        GtkSignalFunc OkCallback, void *OkCbData,
-        GtkSignalFunc CancelCallback, void *CancelCbData)
+        char *WTitle, char *Question, gint modal_flag,
+        char *alt1, char *alt2, char *alt3, char *alt4, char *alt5,
+        GtkSignalFunc AnswerCallback)
 {
-   GtkWidget *frame, *label, *button, *box1, *box2;
+   GtkWidget *frame, *label, *button = NULL, *box1, *box2;
+   DialogAnswer *answer;
+   int i;
 
    *DialogWindow = gtk_window_new(GTK_WINDOW_DIALOG);
    gtk_window_set_wmclass(GTK_WINDOW(*DialogWindow), WmName, WmClass);
    gtk_window_set_title(GTK_WINDOW(*DialogWindow), WTitle);
    gtk_window_set_position(GTK_WINDOW(*DialogWindow), GTK_WIN_POS_CENTER);
    gtk_container_border_width(GTK_CONTAINER(*DialogWindow), 10);
+   gtk_window_set_modal(GTK_WINDOW(*DialogWindow), modal_flag);
+   /* attach AnswerCallback to "destroy" so appropriate actions can be taken */
+   gtk_signal_connect_object(GTK_OBJECT(*DialogWindow), "destroy",
+      (GtkSignalFunc) AnswerCallback, (gpointer)bw);
    gtk_signal_connect(GTK_OBJECT(*DialogWindow), "destroy",
-                      (GtkSignalFunc) Interface_destroy_window, DialogWindow);
+      (GtkSignalFunc) Interface_destroy_window, DialogWindow);
 
    box1 = gtk_vbox_new(FALSE, 5);
    frame = gtk_frame_new(NULL);
@@ -1373,21 +1554,32 @@ static void Interface_make_question_dialog(
    gtk_widget_show(frame);
    gtk_box_pack_start(GTK_BOX(box1), frame, TRUE, TRUE, 0);
 
+   /* This struct will save us some code and lots of callback functions.
+    * (it looks clumsy, but provides a handy interface for the caller) */
+   answer = g_new(DialogAnswer, 6);
+   for (i = 0; i < 6; ++i) {
+      answer[i].bw = bw;
+      answer[i].alt_num = i;
+      answer[i].this = answer;
+   }
+   answer[0].alt_str = NULL; answer[1].alt_str = alt1;
+   answer[2].alt_str = alt2; answer[3].alt_str = alt3;
+   answer[4].alt_str = alt4; answer[5].alt_str = alt5;
+   /* Set the default answer */
+   bw->question_dialog_answer = &answer[0];
+
+   /* pack the alternatives */
    box2 = gtk_hbox_new(TRUE, 5);
-   button = gtk_button_new_with_label("OK");
-   gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
-                             OkCallback, OkCbData);
-   gtk_signal_connect(GTK_OBJECT(button), "clicked",
-                      (GtkSignalFunc) Interface_destroy_window, DialogWindow);
-   gtk_widget_show(button);
-   gtk_box_pack_start(GTK_BOX(box2), button, FALSE, TRUE, 0);
-   button = gtk_button_new_with_label("Cancel");
-   gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
-                             CancelCallback, CancelCbData);
-   gtk_signal_connect(GTK_OBJECT(button), "clicked",
-                      (GtkSignalFunc) Interface_destroy_window, DialogWindow);
-   gtk_widget_show(button);
-   gtk_box_pack_start(GTK_BOX(box2), button, FALSE, TRUE, 0);
+   for (i = 1; i < 6 && answer[i].alt_str; ++i) {
+      button = gtk_button_new_with_label(answer[i].alt_str);
+      gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
+         Interface_question_dialog_set_answer_cb, (gpointer)&answer[i]);
+      gtk_signal_connect(GTK_OBJECT(button), "clicked",
+         (GtkSignalFunc) Interface_destroy_window, DialogWindow);
+      gtk_widget_show(button);
+      gtk_box_pack_start(GTK_BOX(box2), button, FALSE, TRUE, 0);
+   }
+
    gtk_box_pack_start(GTK_BOX(box1), box2, FALSE, FALSE, 0);
    gtk_container_add(GTK_CONTAINER(*DialogWindow), box1);
 
@@ -1401,15 +1593,19 @@ static void Interface_make_question_dialog(
  * Create and show an [OK|Cancel] question dialog
  */
 void a_Interface_question_dialog(
-        BrowserWindow *bw, gchar *QuestionTxt,
-        GtkSignalFunc OkCallback, void *OkCbData,
-        GtkSignalFunc CancelCallback, void *CancelCbData)
+        BrowserWindow *bw, gchar *QuestionTxt, gint modal_flag,
+        char *alt1, char *alt2, char *alt3, char *alt4, char *alt5,
+        GtkSignalFunc AnswerCallback)
 {
    if (!bw->question_dialog_window) {
-      Interface_make_question_dialog(&(bw->question_dialog_window),
-         "question_dialog", "Dillo", "Dillo: Question", QuestionTxt,
-         OkCallback, OkCbData, CancelCallback, CancelCbData);
+      Interface_make_question_dialog(
+         bw,
+         &(bw->question_dialog_window), "question_dialog", "Dillo",
+         "Dillo: Question", QuestionTxt, modal_flag,
+         alt1, alt2, alt3, alt4, alt5,
+         AnswerCallback);
    } else {
+      /* should not reach here */
       gtk_widget_destroy(bw->question_dialog_window);
    }
 }
@@ -1481,6 +1677,9 @@ static void Interface_file_save_url(GtkWidget *widget, BrowserWindow *bw)
       Web->flags |= WEB_Download;
       /* todo: keep track of this client */
       a_Capi_open_url(Web, Interface_save_callback, Web);
+
+      g_free(save_dialog_last_dirname);
+      save_dialog_last_dirname = g_strdup(name);
    }
    a_Url_free(url);
 
@@ -1494,7 +1693,7 @@ static void Interface_file_save_link(GtkWidget *widget, BrowserWindow *bw)
 {
    const gchar *name;
    const DilloUrl *url;
-   gchar *cmd, *buf;
+   gchar *cmd, *buf, *s_url, *s_name;
    FILE *out;
    gint buf_size;
 
@@ -1502,13 +1701,20 @@ static void Interface_file_save_link(GtkWidget *widget, BrowserWindow *bw)
              GTK_FILE_SELECTION(bw->save_link_dialog_window));
    url  = a_Menu_popup_get_url(bw);
 
+   g_free(save_dialog_last_dirname);
+   save_dialog_last_dirname = g_strdup(name);
+
    if (!a_Capi_get_buf(url, &buf, &buf_size)) {
       /* Not cached, ask the downloads server to get it */
+      s_url = a_Misc_stuff_chars(URL_STR(url), "'\"");
+      s_name = a_Misc_stuff_chars(name, "'\"");
       cmd = g_strdup_printf(
                "<dpi cmd='download' url='%s' destination='%s'>\n",
-               URL_STR(url), name);
+               s_url, s_name);
       a_Capi_dpi_send_cmd(NULL, bw, cmd, "downloads", 1);
       g_free(cmd);
+      g_free(s_name);
+      g_free(s_url);
 
    } else {
       /* Cached! Save from the cache */
@@ -1531,11 +1737,26 @@ static void Interface_file_save_link(GtkWidget *widget, BrowserWindow *bw)
  */
 static char *Interface_make_save_name(const DilloUrl *url)
 {
-   gchar *FileName;
+   gchar *FileName, *FilenameWithDir;
 
    if ((FileName = strrchr(URL_PATH(url), '/')))
-      return g_strndup(++FileName, MIN(strlen(FileName), 64));
-   return g_strdup("");
+      FileName = g_strndup(FileName + 1, MIN(strlen(FileName + 1), 80));
+   else
+      FileName = g_strdup("");
+
+   if (save_dialog_last_dirname) {
+      if (*FileName) {
+        gchar *dirpart = g_dirname(save_dialog_last_dirname);
+        FilenameWithDir =
+           g_strconcat(dirpart, G_DIR_SEPARATOR_S, FileName, NULL);
+        g_free(dirpart);
+      } else {
+        FilenameWithDir = g_strdup(save_dialog_last_dirname);
+      }
+      g_free(FileName);
+      return FilenameWithDir;
+   } else
+      return FileName;
 }
 
 /*
@@ -1666,7 +1887,7 @@ static void Interface_search_callback(GtkWidget *widget, BrowserWindow *bw)
    gchar *keyw, *url_str;
 
    keyw = gtk_editable_get_chars(GTK_EDITABLE(bw->search_dialog_entry), 0, -1);
-   if (keyw) { 
+   if (keyw) {
       url_str = Interface_make_search_url(keyw);
       Interface_open_url_string(url_str, bw);
       g_free(url_str);
@@ -1704,7 +1925,7 @@ static void Interface_entry_proxy_passwd(GtkWidget *widget, BrowserWindow *bw)
 {
    gchar *text;
 
-   text = gtk_editable_get_chars(GTK_EDITABLE(bw->proxy_passwd_dialog_entry), 
+   text = gtk_editable_get_chars(GTK_EDITABLE(bw->proxy_passwd_dialog_entry),
                                  0, -1);
    a_Http_set_proxy_passwd(text);
    g_free(text);
@@ -1719,16 +1940,16 @@ static void Interface_entry_proxy_passwd(GtkWidget *widget, BrowserWindow *bw)
 void a_Interface_proxy_passwd_dialog(BrowserWindow *bw)
 {
    GString *buf = g_string_new("");
-   
+
    g_string_sprintf(buf, "Dillo: Enter proxy password for '%s'",
                     prefs.http_proxyuser);
-   
+
    if (!bw->proxy_passwd_dialog_window) {
       Interface_make_dialog(&(bw->proxy_passwd_dialog_window),
          "proxy_passwd_dialog", "Dillo", buf->str,
          &(bw->proxy_passwd_dialog_entry), "", 0, NULL, NULL,
          "OK", (GtkSignalFunc)Interface_entry_proxy_passwd, (void *)bw);
-      if (prefs.transient_dialogs) 
+      if (prefs.transient_dialogs)
          gtk_window_set_transient_for(
             GTK_WINDOW(bw->proxy_passwd_dialog_window),
             GTK_WINDOW(bw->main_window));
