@@ -3,6 +3,20 @@
 
 #include "browser.h"
 
+/*
+ * This struct is used with custom dialogs.
+ * It can be extended to provide extra widgets (e.g. check buttons).
+ * 
+ */
+typedef struct _DialogAnswer DialogAnswer;
+struct _DialogAnswer {
+   BrowserWindow *bw;
+   gint alt_num;
+   char *alt_str;
+   DialogAnswer *this;
+};
+
+
 void a_Interface_init(void);
 void a_Interface_stop(BrowserWindow *bw);
 void a_Interface_clean(BrowserWindow *bw);
@@ -40,9 +54,9 @@ void a_Interface_set_button_sens(BrowserWindow *bw);
 void a_Interface_scroll_popup(GtkWidget *widget);
 
 void a_Interface_question_dialog(
-        BrowserWindow *bw, gchar *QuestionTxt,
-        GtkSignalFunc OkCallback, void *OkCbData,
-        GtkSignalFunc CancelCallback, void *CancelCbData);
+        BrowserWindow *bw, gchar *QuestionTxt, gint modal_flag,
+        char *alt1, char *alt2, char *alt3, char *alt4, char *alt5,
+        GtkSignalFunc AnswerCallback);
 void a_Interface_message_window(const char *title, const char *format, ... );
 void a_Interface_text_window (GtkWidget **text_widget,
                               gchar *title, gchar *wm_class,
