@@ -15,16 +15,20 @@ extern "C" {
 #endif /* __cplusplus */
 
 void a_Nav_push(BrowserWindow *bw, const DilloUrl *url);
+void a_Nav_push_nw(BrowserWindow *bw, const DilloUrl *url);
 void a_Nav_vpush(void *vbw, const DilloUrl *url);
 void a_Nav_back(BrowserWindow *bw);
 void a_Nav_forw(BrowserWindow *bw);
 void a_Nav_home(BrowserWindow *bw);
 void a_Nav_reload(BrowserWindow *bw);
 void a_Nav_init(BrowserWindow *bw);
+void a_Nav_free(BrowserWindow *bw);
 void a_Nav_cancel_expect (BrowserWindow *bw);
-void a_Nav_expect_done(BrowserWindow *bw, DwWidget *dw);
-void a_Nav_remove_top_url(BrowserWindow *bw);
+void a_Nav_expect_done(BrowserWindow *bw);
 void a_Nav_jump_callback(GtkWidget *widget, gpointer client_data, gint NewBw);
+gint a_Nav_stack_ptr(BrowserWindow *bw);
+gint a_Nav_stack_size(BrowserWindow *bw);
+
 
 #ifdef __cplusplus
 }
