@@ -87,8 +87,8 @@ static DilloPlain *Plain_new(BrowserWindow *bw)
    /* Create the font and attribute for the page. */
    font_attrs.name = prefs.fw_fontname;
    font_attrs.size = rint(12.0 * prefs.font_factor);
-   font_attrs.bold = FALSE;
-   font_attrs.italic = FALSE;
+   font_attrs.weight = 400;
+   font_attrs.style = DW_STYLE_FONT_STYLE_NORMAL;
 
    a_Dw_style_init_values (&style_attrs, plain->bw->main_window->window);
    style_attrs.font = a_Dw_style_font_new (&font_attrs);
@@ -162,7 +162,7 @@ static void Plain_write(DilloPlain *plain, void *Buf, gint BufSize, gint Eof)
    Start = (char*)Buf + plain->Start_Ofs;
    MaxBytes = BufSize - plain->Start_Ofs;
    i = len = 0;
-   while( i < MaxBytes ) {
+   while ( i < MaxBytes ) {
       switch ( plain->state ) {
       case ST_SeekingEol:
          if ( Start[i] == '\n' || Start[i] == '\r' )
@@ -194,8 +194,10 @@ static void Plain_write(DilloPlain *plain, void *Buf, gint BufSize, gint Eof)
 
    if ( plain->bw ) {
       gchar completestr[32];
-      sprintf(completestr,"%s%.1f Kb", PBAR_PSTR(prefs.panel_size == 1),
-              (float)plain->Start_Ofs/1024);
+      g_snprintf(
+         completestr, 32, "%s%.1f Kb",
+         PBAR_PSTR(prefs.panel_size == 1),
+         (float)plain->Start_Ofs/1024);
       a_Progressbar_update(plain->bw->progress, completestr, 1);
    }
 }
