@@ -78,13 +78,16 @@ gint a_Web_dispatch_by_type (const char *Type, DilloWeb *Web,
       a_Dw_gtk_scrolled_window_set_dw(
          GTK_DW_SCROLLED_WINDOW(Web->bw->docwin), dw);
 
-      if (URL_POSX(Web->url) || URL_POSY(Web->url))
+      if (URL_POSX(Web->url) || URL_POSY(Web->url)) {
         a_Dw_gtk_scrolled_window_set_scrolling_position(
            GTK_DW_SCROLLED_WINDOW(Web->bw->docwin),
            URL_POSX(Web->url), URL_POSY(Web->url));
-      else
+      } else {
+        gchar *pf = a_Url_decode_hex_str(URL_FRAGMENT_(Web->url));
         a_Dw_gtk_scrolled_window_set_anchor(
-           GTK_DW_SCROLLED_WINDOW(Web->bw->docwin), URL_FRAGMENT_(Web->url));
+           GTK_DW_SCROLLED_WINDOW(Web->bw->docwin), pf);
+        g_free(pf);
+      }
 
       /* Clear the title bar for pages without a <TITLE> tag */
       a_Interface_set_page_title(Web->bw, "");
@@ -98,12 +101,8 @@ gint a_Web_dispatch_by_type (const char *Type, DilloWeb *Web,
 
    } else {
       /* A non-RootUrl. At this moment we only handle image-children */
-      if (!g_strncasecmp(Type, "image/", 6)) {
+      if (!g_strncasecmp(Type, "image/", 6))
          dw = a_Mime_set_viewer(Type, Web, Call, Data);
-      } else {
-         *Call = a_Cache_null_client;
-         *Data = NULL;
-      }
    }
 
    if (!dw) {
